---------------------------------------------------------------------------------------------------
--
--  AUDIO_ROUTE.VHDL
--
--  Routet die Audio-Signale des DAPs auf den 8 Bit DA Wandler
--
--  Autor : Rene Bllhoff
--  Datum : 27.09.2005
--
--  (c) 2005 by Rene Bllhoff
--
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity dap_audio_route is
		port (clk   	: in 	std_logic;
					res 		: in 	std_logic;

					outpwm  : out std_logic;
					smp_clk	: out std_logic;

					outmode : in std_logic_vector  ( 2 downto 0);

					input1	: in 	std_logic_vector (31 downto 0);
					input2	: in 	std_logic_vector (31 downto 0);
					input3	: in 	std_logic_vector (31 downto 0);
					input4	: in 	std_logic_vector (31 downto 0);

					output1 : out std_logic_vector (31 downto 0);
					output2 : out std_logic_vector (31 downto 0);
					output3 : out std_logic_vector (31 downto 0);
					output4 : out std_logic_vector (31 downto 0)
				 );
end dap_audio_route;

architecture Behavioral of dap_audio_route is

	COMPONENT audio_pwm1
	PORT(
		clk50m : IN std_logic;
		res : IN std_logic;
		dataout : IN std_logic_vector(7 downto 0);          
		newdata : OUT std_logic;
		pwmout : OUT std_logic
		);
	END COMPONENT;

	signal output_u  : std_logic_vector (31 downto 0);
	signal outputs_u : std_logic_vector (31 downto 0);

	signal output8	 : std_logic_vector ( 7 downto 0);
	signal output8_t : std_logic_vector ( 7 downto 0);
	signal cnt       : std_logic_vector ( 1 downto 0);
	signal new_smp	 : std_logic;


	type audio_route_states is (audio_route_idle,
															audio_route_start,
															audio_route_mix,
															audio_route_mix_01,
															audio_route_mix_02,
															audio_route_mix_03,
															audio_route_mix_04,
															audio_route_out,
															audio_route_out_01,
															audio_route_out_02
														 );

	signal state : audio_route_states;

begin

	Inst_audio_pwm1: audio_pwm1 PORT MAP(
		clk50m 	=> clk,
		res 		=> res,
		newdata => new_smp,
		dataout => output8,
		pwmout 	=> outpwm
	);

	process (clk, res, 
					 new_smp, state,	

					 input1, input2, input3, input4,
					 output_u, outputs_u, output8, output8_t
					)
	begin 	

		if res = '1' then
			output_u 		<= (others => '0');
			outputs_u 	<= (others => '0');

			output8	  	<= (others => '0');
			output8_t 	<= (others => '0');

			state 	 		<= audio_route_idle;
			--new_smp			<= '0';

		elsif clk'event and clk = '1' then
		
			smp_clk 		<= new_smp;

			if new_smp = '1' then
				state 	 	<= audio_route_start;
				output8 	<= output8_t;
			end if;

			case state is

				when audio_route_idle => NULL;

				when audio_route_start =>		
				
					if outmode (2) = '0' then
						state 	 <= audio_route_out;
					else
						output_u <= (others => '0');
						state 	 <= audio_route_mix;
						cnt			 <= "00";
					end if;

				when audio_route_mix =>
					case cnt is
						when "00" 	=> outputs_u <= input1;
						when "01" 	=> outputs_u <= input2;
						when "10" 	=> outputs_u <= input3;
						when "11" 	=> outputs_u <= input4;
						when others => NULL;
					end case;
					state 	 <= audio_route_mix_01;

				when audio_route_mix_01 =>
				  outputs_u 		<= outputs_u + X"00800000";
					state 	 			<= audio_route_mix_02;

				when audio_route_mix_02 =>
					output_u		  <= output_u + outputs_u;
					cnt 					<= cnt + '1';
					state 	 			<= audio_route_mix_03;

				when audio_route_mix_03 =>
				  if cnt = "00" then
					  state <= audio_route_mix_04;
					else
					  state <= audio_route_mix;
					end if;

				when audio_route_mix_04 =>
					output_u 	<= "00" & output_u (31 downto 2); -- / 4
					state			<= audio_route_out_02;
		
				when audio_route_out =>
					case outmode (1 downto 0) is
						when "00" 	=> outputs_u <= input1;
						when "01" 	=> outputs_u <= input2;
						when "10" 	=> outputs_u <= input3;
						when "11" 	=> outputs_u <= input4;
						when others => NULL;
					end case;
					state 		<= audio_route_out_01;
							
				when audio_route_out_01 =>
					output_u 	<= outputs_u + X"00800000";
					state 		<= audio_route_out_02;
							
				when audio_route_out_02 =>
					output8_t	<= output_u (23 downto 16);
					state 		<= audio_route_idle;

				when others => NULL;

			end case;

		end if;
	
	end process;


end Behavioral;
