---------------------------------------------------------------------------------------------------
--
--  DAP_BASE.VHDL
--
--  Der Audio-Prozessor
--
--  Autor : Rene Bllhoff
--  Datum : 27.09.2005
--
--  (c) 2005 by Rene Bllhoff
--
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity dap_base is
	port (
				clk 				: in 	std_logic;
				res 				: in 	std_logic;

				dapi_trg		: in 	std_logic;
				dapi_rdy		: out	std_logic;
				dapi_sel		: in 	std_logic;
				dapi_din		: in 	std_logic_vector (7 downto 0);
				dapi_dout		: out	std_logic_vector (7 downto 0);
				


				new_sample 	: in 	std_logic;

				input1 			: in  std_logic_vector (31 downto 0);
				input2 			: in  std_logic_vector (31 downto 0);
				input3 			: in  std_logic_vector (31 downto 0);
				input4 			: in  std_logic_vector (31 downto 0);

				output1 		: out std_logic_vector (31 downto 0);
				output2 		: out std_logic_vector (31 downto 0);
				output3 		: out std_logic_vector (31 downto 0);
				output4 		: out std_logic_vector (31 downto 0)
	);
end dap_base;

architecture Behavioral of dap_base is


	----------------------------------------------------------------
	----------------------------------------------------------------
	----
	----
	---- states
	----
	----
	----------------------------------------------------------------
	----------------------------------------------------------------

	type all_states is (
											-------------------------------------------------------
											--
											--  system-states fr DAP-Base
											--
											-------------------------------------------------------

											-- statemachine initialisieren bzw resetten (fr debug und normalen reset)
											init_fsm,
											init_fsm_01,

											reset_fsm,
											reset_fsm_01,

											-- opcode liegt vor -> dekodieren und state setzen
											fetch_opcode, 	 	-- adressen extrahieren und nach state verzweigen

											-- nchste instruktion holen (und debug behandeln)
											next_instruction,			-- pm_adr erhhen
											next_instruction_01,	-- (unbenutzt) fr debug

											-- bei breakpoint warten
											handle_breakpoint,

											-- sample-bearbeitet (abgearbeitet) und auf neues sample warten
											end_process,					-- auf neues sample warten -> pm_adr resetten


											-------------------------------------------------------
											--
											--  "unterprogramme" fr speicher zugriffe
											--
											-------------------------------------------------------

											-- paralleles holen von daten (AMEM, BMEM, DMEM)

											fetch_mult_mem,			-- init
											fetch_mult_mem_01,	-- holen & increment
											fetch_mult_mem_02,	-- check

											put_mult_mem,				-- init
											put_mult_mem_01,		-- schreiben & increment
											put_mult_mem_02,		-- check


											-------------------------------------------------------
											--
											--  audio-prozess-states
											--
											-------------------------------------------------------
													
											-- oscillator																																	 
											ap_osc,				
											ap_osc_01,	
											--ap_osc_01a,	
											ap_osc_02,	
											ap_osc_03,		
											--ap_osc_03a,		
											--ap_osc_03b,		
											ap_osc_04,		

											-- addierer
											ap_add,				
											ap_add_01,	
											ap_add_02,		

											-- verstrken/abschwchen
											ap_amp,				
											ap_amp_01,	
											ap_amp_02,		
											ap_amp_03, 		

											-- audio-werte im speicher verschieben
											ap_move,			-- holen
											ap_move_01,		-- speichern

											-- biquad durchrechnen
											ap_biquad,
											ap_biquad_01,
											ap_biquad_02,
											ap_biquad_03,
											ap_biquad_04,
											ap_biquad_04a,
											ap_biquad_05,
											ap_biquad_06,
											ap_biquad_07,
											ap_biquad_08,
											ap_biquad_09,

											-- ausgabe in temporres sample-register
											ap_output,		-- holen
											ap_output_01,	-- routen

											-- eingabe von temporrem sample-register
											ap_input,			-- routen
											ap_input_01,	-- speichern

											-- clipping 
											ap_clip,
											ap_clip_01,
											ap_clip_02

										 );

	type dapi_states is  (dapi_kind,
												dapi_mem_adr_hi,
												dapi_mem_adr_lo,
												dapi_mem_read,
												dapi_mem_write,
												dapi_debug_adr,
												dapi_debug_read,
												dapi_debug_write
										 	);

	type dapi_mstates is (dapi_midle,
												dapi_mread,
												dapi_mread_01,
												dapi_mread_02,
												dapi_mwrite,
												dapi_mwrite_01,
												dapi_dread,
												dapi_dread_01,
												dapi_dwrite,
												dapi_dwrite_01
												 );

	----------------------------------------------------------------
	----------------------------------------------------------------
	----
	----
	---- signale
	----
	----
	----------------------------------------------------------------
	----------------------------------------------------------------

	--  state machine																	 	

	signal curr_state 	: all_states;
	signal return_state	: all_states;


	-- speicher-interface

	signal dapi_prdata 	: std_logic_vector ( 7 downto 0);
	signal dapi_ardata 	: std_logic_vector ( 7 downto 0);
	signal dapi_brdata 	: std_logic_vector ( 7 downto 0);
	signal dapi_drdata 	: std_logic_vector ( 7 downto 0);

	signal dapi_pen    	: std_logic;
	signal dapi_aen    	: std_logic;
	signal dapi_ben    	: std_logic;
	signal dapi_den    	: std_logic;

	signal dapi_state	 	: dapi_states;
	signal dapi_mstate 	: dapi_mstates;

	signal dapi_adr    	: std_logic_vector (15 downto 0);
	signal dapi_wen    	: std_logic;
	signal dapi_mena   	: std_logic;
	signal dapi_wdata  	: std_logic_vector ( 7 downto 0);

	signal dbi_ena	 	 	: std_logic;
	signal dbi_wdata		: std_logic_vector (7 downto 0);
	signal dbi_rdata		: std_logic_vector (7 downto 0);


	-- programm-speicher

	signal pm_adr 	: std_logic_vector ( 8 downto 0);
	signal pm_data	: std_logic_vector (31 downto 0);

	-- audio-prozess-speicher

	signal am_badr	: std_logic_vector ( 8 downto 0);
	signal am_offs	: std_logic_vector ( 3 downto 0);
	signal am_eadr 	: std_logic_vector ( 8 downto 0);
	signal am_cnt 	: std_logic_vector ( 3 downto 0);
	signal am_dest	: std_logic_vector ( 3 downto 0);
	signal am_rdata : std_logic_vector (31 downto 0);
	signal am_wdata : std_logic_vector (31 downto 0);
	signal am_en		: std_logic											;	
	signal am_wen		: std_logic											;	
	signal am_fe 		: std_logic											;	
	signal am_pe 		: std_logic											;	

	signal bm_badr 	: std_logic_vector  (8 downto 0);
	signal bm_offs 	: std_logic_vector  (3 downto 0);
	signal bm_eadr 	: std_logic_vector  (8 downto 0);
	signal bm_cnt 	: std_logic_vector ( 3 downto 0);
	signal bm_dest	: std_logic_vector ( 3 downto 0);
	signal bm_rdata : std_logic_vector (31 downto 0);
	signal bm_wdata : std_logic_vector (31 downto 0);
	signal bm_en		: std_logic											;
	signal bm_wen		: std_logic											;	
	signal bm_fe		: std_logic											;	
	signal bm_pe 		: std_logic											;	

	signal dm_badr 	: std_logic_vector  (8 downto 0);
	signal dm_offs 	: std_logic_vector  (3 downto 0);
	signal dm_eadr 	: std_logic_vector  (8 downto 0);
	signal dm_cnt 	: std_logic_vector ( 3 downto 0);
	signal dm_dest	: std_logic_vector ( 3 downto 0);
	signal dm_rdata : std_logic_vector (31 downto 0);
	signal dm_wdata : std_logic_vector (31 downto 0);
	signal dm_en		: std_logic											;
	signal dm_wen		: std_logic											;	
	signal dm_fe		: std_logic											;	
	signal dm_pe 		: std_logic											;	

	-- audio-prozess

	-- allgemeine register

	signal adata1 	: signed 						(31 downto 0);
	signal adata2 	: signed 						(31 downto 0);
	signal adata3 	: signed 						(31 downto 0);
	signal adata4 	: signed 						(31 downto 0);

	signal bdata1 	: signed 					 	(31 downto 0);
	signal bdata2 	: signed 					 	(31 downto 0);
	signal bdata3 	: signed 					 	(31 downto 0);
	signal bdata4 	: signed 					 	(31 downto 0);
	
	signal ddata1 	: std_logic_vector 	(31 downto 0);
	signal ddata2 	: std_logic_vector 	(31 downto 0);
	signal ddata3 	: std_logic_vector 	(31 downto 0);
	signal ddata4 	: std_logic_vector 	(31 downto 0);
	signal ddata5 	: std_logic_vector 	(31 downto 0);
	--signal ddata6 	: std_logic_vector 	(31 downto 0);
	--signal ddata7 	: std_logic_vector 	(31 downto 0);
	--signal ddata8 	: std_logic_vector 	(31 downto 0);


	-- multiplizierer

	signal mul1_a		: signed						(31 downto 0);
	signal mul1_b		: signed						(31 downto 0);
	signal mul1_r		: signed						(63 downto 0);

	signal mul2_a		: signed						(31 downto 0);
	signal mul2_b		: signed						(31 downto 0);
	signal mul2_r		: signed						(63 downto 0);

--	signal adds1_a	: signed						(31 downto 0);
--	signal adds1_b	: signed						(31 downto 0);
--	signal adds1_r	: signed						(31 downto 0);

--	signal addu1_a	: std_logic_vector	(31 downto 0);
--	signal addu1_b	: std_logic_vector	(31 downto 0);
--	signal addu1_r	: std_logic_vector	(31 downto 0);

--	signal sub1_a		: signed						(31 downto 0);
--	signal sub1_b		: signed						(31 downto 0);
--	signal sub1_r		: signed						(31 downto 0);


	signal accu_64	: signed						(63 downto 0);


	-- timing-informationen (fr statistik)

	signal active_time_ctr : std_logic_vector (15 downto 0);
	signal active_time     : std_logic_vector (15 downto 0);

	-- debug

	signal debug 					 : std_logic;

	signal bp_adr1				 : std_logic_vector ( 8 downto 0);
	signal bp_adr2				 : std_logic_vector ( 8 downto 0);
	signal bp_adr3				 : std_logic_vector ( 8 downto 0);
	signal bp_adr4				 : std_logic_vector ( 8 downto 0);

	signal bp_ena1				 : std_logic;
	signal bp_ena2				 : std_logic;
	signal bp_ena3				 : std_logic;
	signal bp_ena4				 : std_logic;

	signal bp_hit					 : std_logic_vector ( 2 downto 0);
	signal bp_single			 : std_logic;
	signal bp_run					 : std_logic;

	-- sample steuerung

	signal toutput1 : std_logic_vector (31 downto 0);
	signal toutput2 : std_logic_vector (31 downto 0);
	signal toutput3 : std_logic_vector (31 downto 0);
	signal toutput4 : std_logic_vector (31 downto 0);
	
	signal tinput1 	: std_logic_vector (31 downto 0);
	signal tinput2 	: std_logic_vector (31 downto 0);
	signal tinput3 	: std_logic_vector (31 downto 0);
	signal tinput4 	: std_logic_vector (31 downto 0);


	----------------------------------------------------------------
	----------------------------------------------------------------
	----
	----
	---- Konstanten
	----
	----
	----------------------------------------------------------------
	----------------------------------------------------------------


	-- generelle audio-konstanten

	constant cAUDIO_M10_32 : signed (31 downto 0) := X"FF800000";
	constant cAUDIO_P10_32 : signed (31 downto 0) := X"007FFFFF";
	constant cAUDIO_NUL_32 : signed (31 downto 0) := X"00000000";

	constant cAUDIO_M10_64 : signed (63 downto 0) := X"FF80000000000000";	 -- begrenzung des accus im mac-ausgang (32bit + 24 bit shift)
	constant cAUDIO_P10_64 : signed (63 downto 0) := X"007FFFFFFFFFFFFF";


	-- constanten fr register

	constant areg1 					: std_logic_vector (3 downto 0) := "0000";
	constant areg2 					: std_logic_vector (3 downto 0) := "0001";
	constant areg3 					: std_logic_vector (3 downto 0) := "0010";
	constant areg4 					: std_logic_vector (3 downto 0) := "0011";


	constant breg1 					: std_logic_vector (3 downto 0) := "0000";
	constant breg2 					: std_logic_vector (3 downto 0) := "0001";
	constant breg3 					: std_logic_vector (3 downto 0) := "0010";
	constant breg4 					: std_logic_vector (3 downto 0) := "0011";


	constant dreg1 					: std_logic_vector (3 downto 0) := "0000";
	constant dreg2 					: std_logic_vector (3 downto 0) := "0001";
	constant dreg3 					: std_logic_vector (3 downto 0) := "0010";
	constant dreg4 					: std_logic_vector (3 downto 0) := "0011";
	constant dreg5 					: std_logic_vector (3 downto 0) := "0100";
	constant dreg6 					: std_logic_vector (3 downto 0) := "0101";
	constant dreg7 					: std_logic_vector (3 downto 0) := "0110";
	constant dreg8 					: std_logic_vector (3 downto 0) := "0111";

	-- constanten fr temporre sample-register

	constant tsr_in1 				: std_logic_vector (1 downto 0) := "00";
	constant tsr_in2 				: std_logic_vector (1 downto 0) := "01";
	constant tsr_in3 				: std_logic_vector (1 downto 0) := "10";
	constant tsr_in4 				: std_logic_vector (1 downto 0) := "11";

	constant tsr_out1 			: std_logic_vector (1 downto 0) := "00";
	constant tsr_out2 			: std_logic_vector (1 downto 0) := "01";
	constant tsr_out3 			: std_logic_vector (1 downto 0) := "10";
	constant tsr_out4 			: std_logic_vector (1 downto 0) := "11";
																							 

	-- constanten fr opcodes

	constant opc_nop 				: std_logic_vector (4 downto 0) := "00000"; -- 00
	constant opc_osc 				: std_logic_vector (4 downto 0) := "00001";	-- 01
	constant opc_amp 				: std_logic_vector (4 downto 0) := "00010";	-- 02
	constant opc_add 				: std_logic_vector (4 downto 0) := "00011";	-- 03
	constant opc_output		 	: std_logic_vector (4 downto 0) := "00100";	-- 04 
	constant opc_input  	 	: std_logic_vector (4 downto 0) := "00101";	-- 05
	constant opc_move   	 	: std_logic_vector (4 downto 0) := "00110";	-- 06
	constant opc_filter 	 	: std_logic_vector (4 downto 0) := "00111";	-- 07
	constant opc_clip   	 	: std_logic_vector (4 downto 0) := "01000";	-- 08

	constant opc_halt   	 	: std_logic_vector (4 downto 0) := "11111"; -- 1f
															 
					

	----------------------------------------------------------------
	----------------------------------------------------------------
	----
	----
	---- Komponenten Deklaration
	----
	----
	----------------------------------------------------------------
	----------------------------------------------------------------

	COMPONENT PMEM_RAM
	PORT(
		clkr1 		: IN std_logic;
		clkr2 		: IN std_logic;
		r2addr 		: IN std_logic_vector(8 downto 0);
		r2wdata 	: IN std_logic_vector(31 downto 0);
		r2en 			: IN std_logic;
		r2wen 		: IN std_logic;
		r1addr 		: IN std_logic_vector(10 downto 0);
		r1wdata 	: IN std_logic_vector(7 downto 0);
		r1en 			: IN std_logic;
		r1wen 		: IN std_logic;          
		r2rdata 	: OUT std_logic_vector(31 downto 0);
		r1rdata 	: OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;


	COMPONENT AMEM_RAM
	PORT(
		clkr1 		: IN std_logic;
		clkr2 		: IN std_logic;
		r2addr 		: IN std_logic_vector(8 downto 0);
		r2wdata 	: IN std_logic_vector(31 downto 0);
		r2en 			: IN std_logic;
		r2wen 		: IN std_logic;
		r1addr 		: IN std_logic_vector(10 downto 0);
		r1wdata 	: IN std_logic_vector(7 downto 0);
		r1en 			: IN std_logic;
		r1wen 		: IN std_logic;          
		r2rdata 	: OUT std_logic_vector(31 downto 0);
		r1rdata 	: OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;


	COMPONENT BMEM_RAM
	PORT(
		clkr1 		: IN std_logic;
		clkr2 		: IN std_logic;
		r2addr 		: IN std_logic_vector(8 downto 0);
		r2wdata 	: IN std_logic_vector(31 downto 0);
		r2en 			: IN std_logic;
		r2wen 		: IN std_logic;
		r1addr 		: IN std_logic_vector(10 downto 0);
		r1wdata 	: IN std_logic_vector(7 downto 0);
		r1en 			: IN std_logic;
		r1wen 		: IN std_logic;          
		r2rdata 	: OUT std_logic_vector(31 downto 0);
		r1rdata 	: OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;


	COMPONENT DMEM_RAM
	PORT(
		clkr1 		: IN std_logic;
		clkr2 		: IN std_logic;
		r2addr 		: IN std_logic_vector(8 downto 0);
		r2wdata 	: IN std_logic_vector(31 downto 0);
		r2en 			: IN std_logic;
		r2wen 		: IN std_logic;
		r1addr 		: IN std_logic_vector(10 downto 0);
		r1wdata 	: IN std_logic_vector(7 downto 0);
		r1en 			: IN std_logic;
		r1wen 		: IN std_logic;          
		r2rdata 	: OUT std_logic_vector(31 downto 0);
		r1rdata 	: OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	-- leer signale

begin

	-- block ram instantiierung

	INST_PMEM_RAM: PMEM_RAM PORT MAP(
		clkr1 		=> clk,
		clkr2 		=> clk,
		r2addr 		=> pm_adr,
		r2wdata 	=> (others => '0'),
		r2rdata 	=> pm_data,
		r2en 			=> '1',
		r2wen 		=> '0',
		r1addr 		=> dapi_adr (10 downto 0),
		r1wdata 	=> dapi_wdata,
		r1rdata 	=> dapi_prdata,
		r1en 			=> dapi_pen,
		r1wen 		=> dapi_wen
	);


	INST_AMEM_RAM: AMEM_RAM PORT MAP(
		clkr1 		=> clk,
		clkr2 		=> clk,
		r2addr 		=> am_eadr,
		r2wdata 	=> am_wdata,
		r2rdata 	=> am_rdata,
		r2en 			=> am_en,
		r2wen 		=> am_wen,
		r1addr 		=> dapi_adr (10 downto 0),
		r1wdata 	=> dapi_wdata,
		r1rdata 	=> dapi_ardata,
		r1en 			=> dapi_aen,
		r1wen 		=> dapi_wen
	);


	INST_BMEM_RAM: BMEM_RAM PORT MAP(
		clkr1 		=> clk,
		clkr2 		=> clk,
		r2addr 		=> bm_eadr,
		r2wdata 	=> bm_wdata,
		r2rdata 	=> bm_rdata,
		r2en 			=> bm_en,
		r2wen 		=> bm_wen,
		r1addr 		=> dapi_adr (10 downto 0),
		r1wdata 	=> dapi_wdata,
		r1rdata 	=> dapi_brdata,
		r1en 			=> dapi_ben,
		r1wen 		=> dapi_wen
	);

	INST_DMEM_RAM: DMEM_RAM PORT MAP(
		clkr1 		=> clk,
		clkr2 		=> clk,
		r2addr 		=> dm_eadr,
		r2wdata 	=> dm_wdata,
		r2rdata 	=> dm_rdata,
		r2en 			=> dm_en,
		r2wen 		=> dm_wen,
		r1addr 		=> dapi_adr (10 downto 0),
		r1wdata 	=> dapi_wdata,
		r1rdata 	=> dapi_drdata,
		r1en 			=> dapi_den,
		r1wen 		=> dapi_wen
	);

	----------------------------------------------------------------
	----------------------------------------------------------------
	----
	----
	---- Audio-Prozess-Statemachine
	----
	----
	----------------------------------------------------------------
	----------------------------------------------------------------


	AUDIO_STATEMACHINE : process (clk, res,

					 dapi_trg, dapi_sel, dapi_din,
					 
					 dapi_state, dapi_mstate,
					 dapi_adr, dapi_wdata, dapi_wen,
					 dbi_ena, dbi_rdata, dbi_wdata,

					 pm_adr, pm_data, 
					 am_eadr, bm_eadr, dm_eAdr,
					 am_badr, bm_badr, dm_badr,
					 am_offs, bm_offs, dm_offs,
					 am_cnt, bm_cnt, dm_cnt,
					 am_dest, bm_dest, dm_dest,
					 am_rdata, bm_rdata, dm_rdata,
					 adata1, adata2, adata3, adata4,
					 bdata1, bdata2, bdata3, bdata4,
					 ddata1, ddata2, ddata3, ddata4,

					 tinput1, tinput2, tinput3, tinput4, 
					 toutput1, toutput2, toutput3, toutput4,

					 curr_state, return_state

					)

		--
		--
		-- prozeduren fr unterprogramm-"aufrufe"
		--
		--

		-- paralleles abholen von daten

		procedure fetch_mult_mem 	 (	newstate 	: in all_states;
																	pam_offs	: in std_logic_vector ( 3 downto 0);
																	pam_dest	: in std_logic_vector	( 3 downto 0);
																	pam_cnt		: in std_logic_vector ( 3 downto 0);
																	pbm_offs	: in std_logic_vector ( 3 downto 0);
																	pbm_dest	: in std_logic_vector	( 3 downto 0);
																	pbm_cnt		: in std_logic_vector ( 3 downto 0);
																	pdm_offs	: in std_logic_vector ( 3 downto 0);
																	pdm_dest	: in std_logic_vector	( 3 downto 0);
																	pdm_cnt		: in std_logic_vector ( 3 downto 0)
																) is
		begin
			curr_state 		<= fetch_mult_mem;
			return_state 	<= newstate;
			am_offs				<= pam_offs;
			am_dest				<= pam_dest;
			am_wen 				<= '0';
			am_en 				<= '1';
			am_cnt				<= pam_cnt;

			bm_offs				<= pbm_offs;
			bm_dest				<= pbm_dest;
			bm_wen 				<= '0';
			bm_en 				<= '1';
			bm_cnt				<= pbm_cnt;

			dm_offs				<= pdm_offs;
			dm_dest				<= pdm_dest;
			dm_wen 				<= '0';
			dm_en 				<= '1';
			dm_cnt				<= pdm_cnt;
		end procedure;

		-- paralleles schreiben von daten

		procedure put_mult_mem 	 (	newstate 	: in all_states;
																	pam_offs	: in std_logic_vector ( 3 downto 0);
																	pam_dest	: in std_logic_vector	( 3 downto 0);
																	pam_cnt		: in std_logic_vector ( 3 downto 0);
																	pbm_offs	: in std_logic_vector ( 3 downto 0);
																	pbm_dest	: in std_logic_vector	( 3 downto 0);
																	pbm_cnt		: in std_logic_vector ( 3 downto 0);
																	pdm_offs	: in std_logic_vector ( 3 downto 0);
																	pdm_dest	: in std_logic_vector	( 3 downto 0);
																	pdm_cnt		: in std_logic_vector ( 3 downto 0)
																) is
		begin
			curr_state 		<= put_mult_mem;
			return_state 	<= newstate;
			am_offs				<= pam_offs;
			am_dest				<= pam_dest;
			am_wen 				<= '0';
			am_en 				<= '1';
			am_cnt				<= pam_cnt;

			bm_offs				<= pbm_offs;
			bm_dest				<= pbm_dest;
			bm_wen 				<= '0';
			bm_en 				<= '1';
			bm_cnt				<= pbm_cnt;

			dm_offs				<= pdm_offs;
			dm_dest				<= pdm_dest;
			dm_wen 				<= '0';
			dm_en 				<= '1';
			dm_cnt				<= pdm_cnt;
		end procedure;

		procedure AMEM_DataOut (reg : in std_logic_vector (3 downto 0) ) is
		begin
			case reg is
				when areg1 => am_wdata <= std_logic_vector (adata1);
				when areg2 => am_wdata <= std_logic_vector (adata2);
				when areg3 => am_wdata <= std_logic_vector (adata3);
				when areg4 => am_wdata <= std_logic_vector (adata4);
				when others => NULL;
			end case;
		end procedure;

		procedure AMEM_DataIn (reg : in std_logic_vector (3 downto 0) ) is
		begin
			case reg is
				when areg1 => adata1 <= signed (am_rdata);
				when areg2 => adata2 <= signed (am_rdata);
				when areg3 => adata3 <= signed (am_rdata);
				when areg4 => adata4 <= signed (am_rdata);
				when others => NULL;
			end case;
		end procedure;

		procedure BMEM_DataOut (reg : in std_logic_vector (3 downto 0) ) is
		begin
			case reg is
				when breg1 => bm_wdata <= std_logic_vector (bdata1);
				when breg2 => bm_wdata <= std_logic_vector (bdata2);
				when breg3 => bm_wdata <= std_logic_vector (bdata3);
				when breg4 => bm_wdata <= std_logic_vector (bdata4);
				when others => NULL;
			end case;
		end procedure;

		procedure BMEM_DataIn (reg : in std_logic_vector (3 downto 0) ) is
		begin
			case reg is
				when breg1 => bdata1 <= signed (bm_rdata);
				when breg2 => bdata2 <= signed (bm_rdata);
				when breg3 => bdata3 <= signed (bm_rdata);
				when breg4 => bdata4 <= signed (bm_rdata);
				when others => NULL;
			end case;
		end procedure;


		procedure DMEM_DataOut (reg : in std_logic_vector (3 downto 0) ) is
		begin
			case reg is
				when dreg1 => dm_wdata <= std_logic_vector (ddata1);
				when dreg2 => dm_wdata <= std_logic_vector (ddata2);
				when dreg3 => dm_wdata <= std_logic_vector (ddata3);
				when dreg4 => dm_wdata <= std_logic_vector (ddata4);
				when dreg5 => dm_wdata <= std_logic_vector (ddata5);
				--when dreg6 => dm_wdata <= std_logic_vector (ddata6);
				--when dreg7 => dm_wdata <= std_logic_vector (ddata7);
				--when dreg8 => dm_wdata <= std_logic_vector (ddata8);
				when others => NULL;
			end case;
		end procedure;

		procedure DMEM_DataIn (reg : in std_logic_vector (3 downto 0) ) is
		begin
			case reg is
				when dreg1 => ddata1 <= std_logic_vector (dm_rdata);
				when dreg2 => ddata2 <= std_logic_vector (dm_rdata);
				when dreg3 => ddata3 <= std_logic_vector (dm_rdata);
				when dreg4 => ddata4 <= std_logic_vector (dm_rdata);
				when dreg5 => ddata5 <= std_logic_vector (dm_rdata);
				--when dreg6 => ddata6 <= std_logic_vector (dm_rdata);
				--when dreg7 => ddata7 <= std_logic_vector (dm_rdata);
				--when dreg8 => ddata8 <= std_logic_vector (dm_rdata);
				when others => NULL;
			end case;
		end procedure;

	begin
		if res = '1' then

			----------------------------------------------------------------
			----------------------------------------------------------------
			----
			----
			---- Reset
			----
			----
			----------------------------------------------------------------
			----------------------------------------------------------------


			curr_state 		<= init_fsm;
			return_state 	<= fetch_opcode;

			active_time_ctr	<= (others => '0');
			active_time			<= (others => '0');


			-- speicher interface signale resetten
			 
			pm_adr 				<= (others => '0');
	
			am_badr				<= (others => '0');
			am_offs				<= (others => '0');
			am_cnt 				<= (others => '0');
			am_en					<= '0';
			am_wen				<= '0';
			am_fe					<= '0';
			am_pe					<= '0';
			am_wdata			<= (others => '0');
			am_dest				<= (others => '0');

			bm_badr 			<= (others => '0');
			bm_offs 			<= (others => '0');
			bm_cnt 				<= (others => '0');
			bm_en					<= '0';
			bm_wen				<= '0';
			bm_fe					<= '0';
			bm_pe					<= '0';
			bm_wdata			<= (others => '0');
			bm_dest				<= (others => '0');

			dm_badr				<= (others => '0');
			dm_offs 			<= (others => '0');
			dm_cnt 				<= (others => '0');
			dm_en					<= '0';
			dm_wen				<= '0';
			dm_fe					<= '0';
			dm_pe					<= '0';
			dm_wdata			<= (others => '0');
			dm_dest				<= (others => '0');


			-- audio-prozess-daten resetten

			adata1				<= (others => '0');
			adata2				<= (others => '0');
			adata3				<= (others => '0');
			adata4				<= (others => '0');

			bdata1				<= (others => '0');
			bdata2				<= (others => '0');
			bdata3				<= (others => '0');
			bdata4				<= (others => '0');

			ddata1				<= (others => '0');
			ddata2				<= (others => '0');
			ddata3				<= (others => '0');
			ddata4				<= (others => '0');
			ddata5				<= (others => '0');
			--ddata6				<= (others => '0');
			--ddata7				<= (others => '0');
			--ddata8				<= (others => '0');

			mul1_a				<= (others => '0');
			mul1_b				<= (others => '0');
			mul2_a				<= (others => '0');
			mul2_b				<= (others => '0');

			--addu1_a				<= (others => '0');
			--addu1_b				<= (others => '0');
			--adds1_a				<= (others => '0');
			--adds1_b				<= (others => '0');
			--sub1_a				<= (others => '0');
			--sub1_b				<= (others => '0');

			accu_64				<= (others => '0');

			-- temporre sample-werte lschen

			tinput1				<= (others => '0');
			tinput2				<= (others => '0');
			tinput3				<= (others => '0');
			tinput4				<= (others => '0');

			toutput1			<= (others => '0');
			toutput2			<= (others => '0');
			toutput3			<= (others => '0');
			toutput4			<= (others => '0');

			-- sample-ausgang resetten

			output1				<= (others => '0');
			output2				<= (others => '0');
			output3				<= (others => '0');
			output4				<= (others => '0');

			-- Debug Interface

			debug					<= '0';

			bp_adr1				<= (others => '0');
			bp_adr2				<= (others => '0');
			bp_adr3				<= (others => '0');
			bp_adr4				<= (others => '0');

			bp_ena1				<= '0';
			bp_ena2				<= '0';
			bp_ena3				<= '0';
			bp_ena4				<= '0';

			bp_run				<= '1';
			bp_single			<= '0';

			bp_hit  			<= (others => '0');

			dbi_ena				<= '0';
			dbi_wdata			<= (others => '0');
			dbi_rdata			<= (others => '0');

			dapi_wdata 		<= (others => '0');
			dapi_wen			<= '0';
			dapi_state		<= dapi_kind;
			dapi_mstate		<= dapi_midle;
			dapi_mena 		<= '0';
			dapi_rdy			<= '1';			
			dapi_dout			<= (others => '0');
			dapi_adr	 		<= (others => '0');


		elsif clk'event and clk = '1' then

			-- timing informationen

			if curr_state /= end_process and bp_run = '1' then
				active_time_ctr <= active_time_ctr + '1';
			end if;


			----------------------------------------------------------------
			----------------------------------------------------------------
			----
			----
			---- Debug-Interface
			----
			----
			----------------------------------------------------------------
			----------------------------------------------------------------


			if dapi_sel = '1' then
			  dapi_rdy <= '0';
				if dapi_trg = '1' then
					case dapi_state is
						when dapi_kind => 
							case dapi_din (3 downto 0) is
								when X"0" 	=> dapi_wen 	<= '1';
															 dapi_state <= dapi_mem_adr_hi;
															 dapi_rdy 	<= '0';
								when X"1" 	=> dapi_wen 	<= '0';
															 dapi_state <= dapi_mem_adr_hi;
															 dapi_rdy 	<= '0';
								when X"2" 	=> dapi_wen 	<= '1';
															 dapi_state <= dapi_debug_adr;											 
															 dapi_rdy 	<= '0';
								when X"3" 	=> dapi_wen 	<= '0';
															 dapi_state <= dapi_debug_adr;											 
															 dapi_rdy 	<= '0';
								when others => NULL;
							end case;

						when dapi_mem_adr_hi =>
						  dapi_adr (15 downto 8) <= dapi_din;
						 	dapi_state 	<= dapi_mem_adr_lo;											 
						 	dapi_rdy 		<= '0';

						when dapi_mem_adr_lo =>
						  dapi_adr ( 7 downto 0) <= dapi_din;
						 	dapi_rdy 		<= '0';
							if dapi_wen = '1' then
								dapi_state 	<= dapi_mem_write;
							else
								dapi_state  <= dapi_mem_read;
								dapi_mstate <= dapi_mread;
							end if;

						when dapi_mem_write =>
							dapi_wdata 	<= dapi_din;
							dapi_mstate <= dapi_mwrite;
							dapi_state  <= dapi_kind;
							dapi_rdy 		<= '1';

						when dapi_mem_read =>
							dapi_state  <= dapi_kind;
							dapi_rdy 		<= '1';

						when dapi_debug_adr => 
						  dapi_adr (3 downto 0)	<= dapi_din (3 downto 0);
							dapi_rdy 	<= '0';
							if dapi_wen = '1' then
								dapi_state  <= dapi_debug_write;
							else
							  dapi_state  <= dapi_debug_read;
								dapi_mstate <= dapi_dread;
							end if;
							
						when dapi_debug_write =>
						  dbi_wdata 	<= dapi_din;
							dapi_mstate <= dapi_dwrite;
							dapi_state  <= dapi_kind;
							dapi_rdy 		<= '1';

						when dapi_debug_read =>
							dapi_state  <= dapi_kind;
							dapi_rdy 		<= '1';

					end case;				  
				end if;
			else
				dapi_state 	<= dapi_kind;
				dapi_rdy 		<= '1';
				dapi_mstate	<= dapi_midle;
			end if;


			case dapi_mstate is
			  when dapi_midle => NULL;

				when dapi_mwrite =>
					dapi_mena 	<= '1';
				  dapi_mstate <= dapi_mwrite_01;

				when dapi_mwrite_01 =>
					dapi_mena 	<= '0';
					dapi_mstate <= dapi_midle;

				when dapi_mread =>
				  dapi_mena 	<= '1';
				  dapi_mstate <= dapi_mread_01;

				when dapi_mread_01 =>
				  dapi_mena 	<= '1';
				  dapi_mstate <= dapi_mread_02;

				when dapi_mread_02 =>
					dapi_mena 	<= '0';
					case dapi_adr (15 downto 12) is
						when X"0" 	=> dapi_dout <= dapi_prdata;
						when X"2" 	=> dapi_dout <= dapi_ardata;
						when X"4" 	=> dapi_dout <= dapi_brdata;
						when X"6" 	=> dapi_dout <= dapi_drdata;
						when others => NULL;
					end case;
				  dapi_mstate <= dapi_midle;

				when dapi_dwrite =>
				  dbi_ena 		<= '1';
				  dapi_mstate <= dapi_dwrite_01;

				when dapi_dwrite_01 =>
				  dbi_ena 		<= '0';
				  dapi_mstate <= dapi_midle;

				when dapi_dread =>
				  dbi_ena 		<= '1';
				  dapi_mstate <= dapi_dread_01;

				when dapi_dread_01 =>
				  dbi_ena 		<= '0';
					dapi_dout 	<= dbi_rdata;
				  dapi_mstate <= dapi_midle;

				when others => NULL;

			end case;


			if dbi_ena = '1' then
				if dapi_wen = '1' then
				  case dapi_adr (3 downto 0) is
						when X"0" => 	debug 									<= dbi_wdata (0);
												 	bp_single 							<= dbi_wdata (1);
												 	bp_run									<= dbi_wdata (2);
						when X"1" =>  bp_ena1									<= dbi_wdata (0);
													bp_ena2									<= dbi_wdata (1);
													bp_ena3									<= dbi_wdata (2);
													bp_ena4									<= dbi_wdata (3);
						when X"2" => 	bp_adr1	( 7 downto  0) 	<= dbi_wdata;
						when X"3" => 	bp_adr1	( 8)          	<= dbi_wdata (0);
						when X"4" => 	bp_adr2	( 7 downto  0) 	<= dbi_wdata;
						when X"5" => 	bp_adr2	( 8)          	<= dbi_wdata (0);
						when X"6" => 	bp_adr3	( 7 downto  0) 	<= dbi_wdata;
						when X"7" => 	bp_adr3	( 8)          	<= dbi_wdata (0);
						when X"8" => 	bp_adr4	( 7 downto  0) 	<= dbi_wdata;
						when X"9" => 	bp_adr4	( 8)          	<= dbi_wdata (0);
						when others => NULL;
					end case;
			  else
				  case dapi_adr (3 downto 0) is
						-- debug control
						when X"0" => 	dbi_rdata (7 downto 6)  <= (others => '0');
												 	dbi_rdata (5 downto 3)  <= bp_hit;
												 	dbi_rdata (2)						<= bp_run;
												 	dbi_rdata (1)						<= bp_single;
												 	dbi_rdata (0)						<= debug;
						when X"1" =>  dbi_rdata (0)						<= bp_ena1;
													dbi_rdata (1)						<= bp_ena2;	
													dbi_rdata (2)						<= bp_ena3;	
													dbi_rdata (3)						<= bp_ena4;	

						-- breakpoint control
						when X"2" => 	dbi_rdata								<= bp_adr1 ( 7 downto 0);
						when X"3" => 	dbi_rdata (7 downto 1)  <= (others => '0');
												 	dbi_rdata	(0)						<= bp_adr1 (8);
						when X"4" => 	dbi_rdata								<= bp_adr2 ( 7 downto 0);
						when X"5" => 	dbi_rdata (7 downto 1)  <= (others => '0');
												 	dbi_rdata	(0)						<= bp_adr2 (8);
						when X"6" => 	dbi_rdata								<= bp_adr3 ( 7 downto 0);
						when X"7" => 	dbi_rdata (7 downto 1)  <= (others => '0');
												 	dbi_rdata	(0)						<= bp_adr3 (8);
						when X"8" => 	dbi_rdata								<= bp_adr4 ( 7 downto 0);
						when X"9" => 	dbi_rdata (7 downto 1)  <= (others => '0');
									 			 	dbi_rdata	(0)						<= bp_adr4 (8);

						-- program counter read
						when X"a" => 	dbi_rdata (7 downto 1)  <= (others => '0');
									 			 	dbi_rdata	(0)						<= pm_adr (8);
						when X"b" => 	dbi_rdata (7 downto 1)  <= (others => '0');
									 			 	dbi_rdata	(0)						<= pm_adr (8);

						-- time measurement read
						when X"c" => 	dbi_rdata								<= active_time_ctr (15 downto  8);
						when X"d" => 	dbi_rdata								<= active_time_ctr ( 7 downto  0);
						when X"e" => 	dbi_rdata								<= active_time 		 (15 downto  8);
						when X"f" => 	dbi_rdata								<= active_time 		 ( 7 downto  0);

						when others => NULL;
					end case;

				end if;
			end if;		

		

			----------------------------------------------------------------
			----------------------------------------------------------------
			----
			----
			---- Audio-Statemachine
			----
			----
			----------------------------------------------------------------
			----------------------------------------------------------------

			case curr_state is

				
				---------------------------------------------------------------
			 	--
				-- state state-machine initialisieren
				--
				---------------------------------------------------------------

				when	init_fsm =>

					pm_Adr <= (others => '0');

					curr_state 	<= init_fsm_01;

				
				when	init_fsm_01 =>

					curr_state 	<= fetch_opcode;

				
				---------------------------------------------------------------
			 	--
				-- state state-machine resetten
				--
				---------------------------------------------------------------
				
				when	reset_fsm =>

					pm_Adr 				<= (others => '0');
					adata1 				<= (others => '0');
					adata2 				<= (others => '0');
					adata3 				<= (others => '0');
					adata4 				<= (others => '0');

					bdata1 				<= (others => '0');
					bdata2 				<= (others => '0');
					bdata3 				<= (others => '0');
					bdata4 				<= (others => '0');

					ddata1 				<= (others => '0');
					ddata2 				<= (others => '0');
					ddata3 				<= (others => '0');
					ddata4 				<= (others => '0');
					ddata5 				<= (others => '0');
					--ddata6 				<= (others => '0');
					--ddata7 				<= (others => '0');
					--ddata8 				<= (others => '0');

					tinput1	 			<= (others => '0');
					tinput2				<= (others => '0');
					tinput3				<= (others => '0');
					tinput4				<= (others => '0');

					toutput1 			<= (others => '0');
					toutput2			<= (others => '0');
					toutput3			<= (others => '0');
					toutput4			<= (others => '0');

					mul1_a				<= (others => '0');
					mul1_b				<= (others => '0');
					mul2_a				<= (others => '0');
					mul2_b				<= (others => '0');

					accu_64				<= (others => '0');

					curr_state 		<= reset_fsm_01;
					return_state 	<= fetch_opcode;

				
				when	reset_fsm_01 =>

					curr_state 	<= fetch_opcode;



				---------------------------------------------------------------
			 	--
				-- state instruktion dekodieren und nach neuen state-verzweigen
				--
				---------------------------------------------------------------
				
				when	fetch_opcode =>

--					optimale ausnutzung des opcodes
--					am_badr			<= pm_data (26 downto 18);
--					bm_badr			<= pm_data (17 downto  9);
--					dm_badr			<= pm_data ( 8 downto  0);

-- 				ausnutzung des opcodes zur besseren lesbarkeit (test und entwicklung)	
					am_badr			<= '0' & pm_data (23 downto 16);
					bm_badr			<= '0' & pm_data (15 downto  8);
					dm_badr			<= '0' & pm_data ( 7 downto  0);

					case pm_data (28 downto 24) is   -- opcode -> state verzweigen
						
						-- nop
						when opc_nop 	=>
							curr_state <= next_instruction;

						-- oscillator 
						when opc_osc =>
							curr_state <= ap_osc;

						-- add
						when opc_add =>
							curr_state <= ap_add;

						-- amp
						when opc_amp =>
							curr_state <= ap_amp;

						-- move
						when opc_move =>
							curr_state <= ap_move;

						-- filter
						when opc_filter =>
							curr_state <= ap_biquad;
						
						-- output
						when opc_output =>
							curr_state <= ap_output;

						-- input
						when opc_input =>
							curr_state <= ap_input;
	
						-- clipping
						when opc_clip =>
							curr_state <= ap_clip;

						-- halt
						when opc_halt =>
							curr_state <= end_process;
					
						-- unimplementiert oder unbenutzt

						when others =>
							curr_state <= next_instruction;
					
					end case;
				     
				---------------------------------------------------------------
			 	--
				-- unterprogramm "fetch_mult_mem"
				--
				---------------------------------------------------------------

				when fetch_mult_mem =>
					if am_cnt = X"0" then
					  am_fe <= '0';
					else 
					  am_fe <= '1';
					end if;    
					if bm_cnt = X"0" then
					  bm_fe <= '0';
					else 
					  bm_fe <= '1';
					end if;    
					if dm_cnt = X"0" then
					  dm_fe <= '0';
					else 
					  dm_fe <= '1';
					end if;    
				  	curr_state 	<= fetch_mult_mem_01;

				when fetch_mult_mem_01 =>
				  if am_fe = '1' then
						AMEM_DataIn (am_dest);
					  am_cnt 			  <= am_cnt - '1';
					  am_offs 		  <= am_offs + '1';
					  am_dest			  <= am_dest + '1';			
				  end if;

				  if bm_fe = '1' then
						BMEM_DataIn (bm_dest);
					  bm_cnt 			  <= bm_cnt - '1';
					  bm_offs 		  <= bm_offs + '1';
					  bm_dest			  <= bm_dest + '1';			
				  end if;

				  if dm_fe = '1' then
						DMEM_DataIn (dm_dest);
					  dm_cnt 			  <= dm_cnt - '1';
					  dm_offs 		  <= dm_offs + '1';
					  dm_dest			  <= dm_dest + '1';			
				  end if;
					
					curr_state 	<= fetch_mult_mem_02;
  
				   
				 when fetch_mult_mem_02 =>
					if am_cnt = X"0" then
						am_fe 	<= '0';
						am_en		<= '0';
					end if;
					if bm_cnt = X"0" then
						bm_fe 	<= '0';
						bm_en		<= '0';
					end if;
					if dm_cnt = X"0" then
						dm_fe 	<= '0';
						dm_en 	<= '0';
					end if;
						
					if am_cnt = X"0" and bm_cnt = X"0" and dm_cnt = X"0" then
						curr_state 	<= return_state;
					else
						curr_state 	<= fetch_mult_mem_01;
          end if;									     


				     
				---------------------------------------------------------------
			 	--
				-- unterprogramm "put_mult_mem"
				--
				---------------------------------------------------------------

				when put_mult_mem =>
					if am_cnt = X"0" then
					  am_pe <= '0';
					else 
					  am_pe <= '1';
					end if;    
					if bm_cnt = X"0" then
					  bm_pe <= '0';
					else 
					  bm_pe <= '1';
					end if;    
					if dm_cnt = X"0" then
					  dm_pe <= '0';
					else 
					  dm_pe <= '1';
					end if;    
				  	curr_state 	<= put_mult_mem_01;

				when put_mult_mem_01 =>
				  if am_pe = '1' then
					  AMEM_DataOut (am_dest);
						am_wen				<= '1';
						am_cnt 			  <= am_cnt - '1';
					  am_dest			  <= am_dest + '1';		
					else
					  am_wen				<= '0';		
				  end if;

				  if bm_pe = '1' then
						BMEM_DataOut (bm_dest);
					  bm_wen				<= '1';
					  bm_cnt 			  <= bm_cnt - '1';

					  bm_dest			  <= bm_dest + '1';			
				  else
					  bm_wen				<= '0';
					end if;

				  if dm_pe = '1' then
						DMEM_DataOut (dm_dest);
					  dm_wen				<= '1';
					  dm_cnt 			  <= dm_cnt - '1';
					  dm_dest			  <= dm_dest + '1';			
					else
					  dm_wen				<= '0';
				  end if;
					curr_state 	<= put_mult_mem_02;
				   
				 when put_mult_mem_02 =>
				  am_wen <= '0';
				  bm_wen <= '0';				  
					dm_wen <= '0';
					if am_cnt = X"0" then
						am_pe 	<= '0';
						am_en		<= '0';
					else
					  am_offs 		  <= am_offs + '1';
					end if;
					if bm_cnt = X"0" then
						bm_pe 	<= '0';
						bm_en		<= '0';
					else
					  bm_offs 		  <= bm_offs + '1';
					end if;
					
					if dm_cnt = X"0" then
						dm_pe 	<= '0';
						dm_en 	<= '0';
					else
					  dm_offs 		  <= dm_offs + '1';
					end if;
						
					if am_cnt = X"0" and bm_cnt = X"0" and dm_cnt = X"0" then
						curr_state 	<= return_state;
					else
						curr_state 	<= put_mult_mem_01;
          end if;									     


				---------------------------------------------------------------
			 	--
				-- state "next_instruction"
				--
				---------------------------------------------------------------

				when next_instruction =>

					if pm_adr /= X"FF" then
						pm_adr <= pm_adr + '1';
						curr_state <= next_instruction_01;
					else
						curr_state <= end_process;
					end if;

				when next_instruction_01 =>
					-- debug handling

				  if debug = '1' then
						bp_hit <= "000";
						if pm_adr = bp_adr1 and bp_ena1 = '1' then
							bp_hit <= "100";
							bp_run <= '0';
							curr_state <= handle_breakpoint;
						elsif pm_adr = bp_adr2 and bp_ena2 = '1' then
							bp_hit <= "101";
							bp_run <= '0';
							curr_state <= handle_breakpoint;
						elsif pm_adr = bp_adr3 and bp_ena3 = '1' then
							bp_hit <= "110";
							bp_run <= '0';
							curr_state <= handle_breakpoint;
						elsif pm_adr = bp_adr4 and bp_ena4 = '1' then
							bp_hit <= "111";
							bp_run <= '0';
							curr_state <= handle_breakpoint;
						else
						  if bp_single = '1' then
								bp_run <= '0';
								curr_state <= handle_breakpoint;
							else
  							bp_run <= '1';
							  curr_state <= fetch_opcode;
							end if;
						end if;
					else
						curr_state <= fetch_opcode;
					end if;

				---------------------------------------------------------------
			 	--
				-- state "handle_breakpoint" (warten auf bp_run)
				--
				---------------------------------------------------------------
						
			 	when handle_breakpoint =>
					if bp_run = '1' then
						curr_state <= fetch_opcode;
					else
						curr_state <= handle_breakpoint;
					end if;

				---------------------------------------------------------------
			 	--
				-- state "end_process" (warten auf neue sample-anfrage)
				--
				---------------------------------------------------------------

				when end_process =>

					-- programm-zhler resetten
          pm_adr 					<= (others => '0');	
					active_time 		<= active_time_ctr;	

					if new_sample = '1' then
		
						-- neu starten

						curr_state <= fetch_opcode;

						-- berechnete samples ausgeben und eingangs-samples bernehmen

						tinput1 <= input1;
						tinput2 <= input2;
						tinput3 <= input3;
						tinput4 <= input4;

						output1 <= toutput1;
						output2 <= toutput2;
						output3 <= toutput3;
						output4 <= toutput4;

						active_time_ctr <= (others => '0');	

					else
					  curr_state <= end_process;
					end if;


				---------------------------------------------------------------
				---------------------------------------------------------------
			 	----
			 	----
				---- 	audio-prozesse
				----
			 	----
				---------------------------------------------------------------
				---------------------------------------------------------------

				---------------------------------------------------------------
			 	--
				-- oscillator 
				--
				---------------------------------------------------------------

				-- 3 states daten holen
				when ap_osc =>
				  fetch_mult_mem (ap_osc_01,
													X"0", areg1, X"0",
													X"0", breg1, X"0",
													X"0", dreg1, X"4");

				-- zhler erhhen
				when ap_osc_01 =>
					ddata4 <= ddata4 + ddata1;
					--addu1_a <= ddata4;
					--addu1_a <= ddata1;
				  --curr_state <= ap_osc_01a;
					curr_state <= ap_osc_02;

				--when ap_osc_01a =>
					--ddata4 <= addu1_r;
				  --curr_state <= ap_osc_02;

				-- zhler abschneiden (8bit)
				when ap_osc_02 =>
					ddata5 (31 downto 24) <= (others => '0');
					ddata5 (23 downto  0) <= ddata4 (31 downto 8);
				  curr_state <= ap_osc_03;

				-- wellenform berechnen
				when ap_osc_03 =>
					case ddata2 (1 downto 0) is
						when "00" 		=> 	adata2 <= signed (ddata5 - X"00800000");					
															--curr_state <= ap_osc_04;
						when "01" 		=> 	if ddata5 > ddata3 then
													 			adata2 <= cAUDIO_M10_32;
																--curr_state <= ap_osc_04;
												 			else
													 			adata2 <= cAUDIO_P10_32;
																--curr_state <= ap_osc_04;
												 			end if;	
						when "10" 		=> 	if ddata5 (23) = '1' then
																adata2  <= cAUDIO_P10_32 - signed ("00000000" & (ddata5 (22 downto 0) & '0'));
																--sub1_a <= cAUDIO_P10_32;
																--sub1_b <= signed ("00000000" & (ddata5 (22 downto 0) & '0'));
																--curr_state <= ap_osc_03b;
															else
																adata2  <= cAUDIO_M10_32 + signed ("00000000" & (ddata5 (22 downto 0) & '0'));
															  --adds1_a <=	cAUDIO_M10_32;
															  --adds1_b <=	signed ("00000000" & (ddata5 (22 downto 0) & '0'));
																--curr_state <= ap_osc_03a;
															end if;
						when others 	=> 	adata2     <= cAUDIO_NUL_32;
															--curr_state <= ap_osc_04;
					end case;
					curr_state <= ap_osc_04;

				--when ap_osc_03a =>
					--adata2 			<= adds1_r;
					--curr_state 	<= ap_osc_04;

				--when ap_osc_03b =>
					--adata2 			<= sub1_r;
					--curr_state 	<= ap_osc_04;

				-- zhlerstand und sample speichern
				when ap_osc_04 =>
					put_mult_mem (next_instruction,
												X"0", areg2, X"1",
												X"0", breg1, X"0",
												X"3", dreg4, X"1");

				---------------------------------------------------------------
			 	--
				-- add 
				--
				---------------------------------------------------------------

				-- 2 audio signale holen
				when ap_add =>
				  fetch_mult_mem (ap_add_01,
													X"0", areg1, X"2",
													X"0", breg1, X"0",
													X"0", dreg1, X"0");

				-- addieren
				when ap_add_01 =>
					adata1 <= adata1 + adata2;
				  curr_state <= ap_add_02;

				-- speichern
				when ap_add_02 =>
					put_mult_mem (next_instruction,
												X"2", areg1, X"1",
												X"0", breg1, X"0",
												X"0", dreg1, X"0");


				---------------------------------------------------------------
			 	--
				-- move 
				--
				---------------------------------------------------------------

				-- audio signal holen
				when ap_move =>
				  fetch_mult_mem (ap_move_01,
													X"0", areg1, X"1",
													X"0", breg1, X"0",
													X"0", dreg1, X"0");

				-- und speichern (achtung speicher adresse in dm_adr !!)
				when ap_move_01 =>
				  am_badr <= dm_badr;
					put_mult_mem (next_instruction,
												X"0", areg1, X"1",
												X"0", breg1, X"0",
												X"0", dreg1, X"0");

				---------------------------------------------------------------
			 	--
				-- output 
				--
				---------------------------------------------------------------

				-- audio wert holen
				when ap_output =>
				  fetch_mult_mem (ap_output_01,
													X"0", areg1, X"1",
													X"0", breg1, X"0",
													X"0", dreg1, X"0");

				-- auf temporren ausgang routen
				when ap_output_01 =>
					case dm_badr (1 downto 0) is
						when tsr_out1 => toutput1 <= std_logic_vector (adata1);
						when tsr_out2 => toutput2 <= std_logic_vector (adata1);
						when tsr_out3 => toutput3 <= std_logic_vector (adata1);
						when tsr_out4 => toutput4 <= std_logic_vector (adata1);
						when others => NULL;
					end case;
				  curr_state <= next_instruction;


				---------------------------------------------------------------
			 	--
				-- input 
				--
				---------------------------------------------------------------

				-- temporres einganssample holen (in die audio-register)
				when ap_input =>
					case dm_badr (1 downto 0) is
						when tsr_in1 => adata1 <= signed (tinput1);
						when tsr_in2 => adata1 <= signed (tinput2);
						when tsr_in3 => adata1 <= signed (tinput3);
						when tsr_in4 => adata1 <= signed (tinput4);
						when others => NULL;
					end case;
					curr_state <= ap_input_01;

				-- in amem speichern (unter am_adr)
				when ap_input_01 =>
					put_mult_mem (next_instruction,
												X"0", areg1, X"1",
												X"0", breg1, X"0",
												X"0", dreg1, X"0");



				---------------------------------------------------------------
			 	--
				-- amp 
				--
				---------------------------------------------------------------

				-- 2 audio signale holen
				when ap_amp =>
				  fetch_mult_mem (ap_amp_01,
													X"0", areg1, X"1",
													X"0", breg1, X"0",
													X"0", dreg1, X"1");

				-- addieren
				when ap_amp_01 =>
					mul1_a			<= adata1;
					mul1_b			<= signed (ddata1);
				  curr_state 	<= ap_amp_02;

				-- speichern
				when ap_amp_02 =>
					adata1 			<= mul1_r (55 downto 24);
				  curr_state 	<= ap_amp_03;

				when ap_amp_03 =>
					put_mult_mem (next_instruction,
												X"1", areg1, X"1",
												X"0", breg1, X"0",
												X"0", dreg1, X"0");


				---------------------------------------------------------------
			 	--
				-- filter 
				--
				---------------------------------------------------------------

				when ap_biquad =>
				  accu_64  		<= (others => '0');
				  fetch_mult_mem (ap_amp_01,
													X"0", areg1, X"1",
													X"0", breg1, X"4",
													X"0", dreg1, X"5");

				when ap_biquad_01 =>
					mul1_a			<= adata1;
					mul1_b			<= signed (ddata1);

					mul2_a			<= bdata3;
					mul2_b			<= signed (ddata4);

				  curr_state 	<= ap_biquad_02;

				when ap_biquad_02 =>
					accu_64 		<= 						mul1_r + mul2_r;
				  curr_state 	<= ap_biquad_03;


				when ap_biquad_03 =>
					mul1_a			<= bdata1;
					mul1_b			<= signed (ddata2);

					mul2_a			<= bdata4;
					mul2_b			<= signed (ddata5);

				  curr_state 	<= ap_biquad_04;

				when ap_biquad_04 =>
					accu_64 		<= accu_64 + 	mul1_r;
				  curr_state 	<= ap_biquad_04a;

				when ap_biquad_04a =>
					accu_64 		<= accu_64 + 	mul2_r;
				  curr_state 	<= ap_biquad_05;
				
				when ap_biquad_05 =>
					mul1_a			<= bdata2;
					mul1_b			<= signed (ddata3);

				  curr_state 	<= ap_biquad_06;

				when ap_biquad_06 =>
					accu_64 		<= accu_64 + 	mul1_r;
				  curr_state 	<= ap_biquad_07;

				when ap_biquad_07 =>
					adata2			<= accu_64 (55 downto 24);

					bdata2			<= bdata1;
					bdata4			<= bdata3;
					curr_state 	<= ap_biquad_08;

				when ap_biquad_08 =>

					bdata1			<= adata1;
					bdata3			<= adata2;
					curr_state 	<= ap_biquad_09;

				when ap_biquad_09 =>
					put_mult_mem (next_instruction,
												X"1", areg2, X"1",
												X"0", breg1, X"4",
												X"0", dreg1, X"0");

				---------------------------------------------------------------
			 	--
				-- clipping
				--
				---------------------------------------------------------------

				when ap_clip =>
				  fetch_mult_mem (ap_clip_01,
													X"0", areg1, X"1",
													X"0", breg1, X"0",
													X"0", dreg1, X"0");

				when ap_clip_01 =>
				     if adata1 > cAUDIO_P10_32 then
							 adata2 <= cAUDIO_P10_32;
					elsif adata1 < cAUDIO_M10_32 then
							 adata2 <= cAUDIO_M10_32;
					else
							 adata2 <= adata1;
					end if;
					curr_state 	<= ap_clip_02;

				when ap_clip_02 =>
					put_mult_mem (next_instruction,
												X"1", areg2, X"1",
												X"0", breg1, X"0",
												X"0", dreg1, X"0");


				---------------------------------------------------------------
			 	--
				-- state-machine ende
				--
				---------------------------------------------------------------

			end case;

		end if;


		-- funktionen auerhalb der State-Machine

		-- Adress-Addierer

		am_eadr <= am_badr + am_offs;
		bm_eadr <= bm_badr + bm_offs;
		dm_eadr <= dm_badr + dm_offs;

		-- multiplizierer

		mul1_r  <= mul1_a  * mul1_b ;
		mul2_r  <= mul2_a  * mul2_b ;

		--adds1_r <= adds1_a + adds1_b;
		--addu1_r <= addu1_a + addu1_b;
		--sub1_r  <= sub1_a  + sub1_b ;

		-- mux fr speicher interface

		if dapi_mena = '1' then
			case dapi_adr (15 downto 12) is
				when X"0"		=> dapi_pen <= '1'; dapi_aen <= '0'; dapi_ben <= '0'; dapi_den <= '0'; 
				when X"2"		=> dapi_pen <= '0'; dapi_aen <= '1'; dapi_ben <= '0'; dapi_den <= '0'; 
				when X"4" 	=> dapi_pen <= '0'; dapi_aen <= '0'; dapi_ben <= '1'; dapi_den <= '0'; 
				when X"6"		=> dapi_pen <= '0'; dapi_aen <= '0'; dapi_ben <= '0'; dapi_den <= '1'; 
				when others => dapi_pen <= '0'; dapi_aen <= '0'; dapi_ben <= '0'; dapi_den <= '0';
			end case;
		else
			dapi_pen <= '0'; dapi_aen <= '0'; dapi_ben <= '0'; dapi_den <= '0';
		end if;
		
	end process;

end Behavioral;


