library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity dap_base_tb1 is
end dap_base_tb1;

architecture Behavioral of dap_base_tb1 is
    
    
	COMPONENT dap_base
	PORT(
		clk : IN std_logic;
		res : IN std_logic;
		dbi_ena : IN std_logic;
		dbi_rw : IN std_logic;
		dbi_adr : IN std_logic_vector(3 downto 0);
		dbi_wdata : IN std_logic_vector(7 downto 0);
		new_sample : IN std_logic;
		input1 : IN std_logic_vector(31 downto 0);
		input2 : IN std_logic_vector(31 downto 0);
		input3 : IN std_logic_vector(31 downto 0);
		input4 : IN std_logic_vector(31 downto 0);          
		dbi_rdata : OUT std_logic_vector(7 downto 0);
		output1 : OUT std_logic_vector(31 downto 0);
		output2 : OUT std_logic_vector(31 downto 0);
		output3 : OUT std_logic_vector(31 downto 0);
		output4 : OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;    
    
   signal clk       : std_logic;    
   signal res       : std_logic;
   signal sampleclk : std_logic;
   
   signal dbi_ena   : std_logic;
   signal dbi_rw    : std_logic;
   signal dbi_adr   : std_logic_vector (3 downto 0);
   signal dbi_rdata : std_logic_vector (7 downto 0);
   signal dbi_wdata : std_logic_vector (7 downto 0);
   
	constant MCLOCK_PER   : time := 20 ns;
	constant SMPCLOCK_PER : time := 20 us;   
    
   signal output1 : std_logic_vector (31 downto 0); 
   signal output2 : std_logic_vector (31 downto 0);
   signal output3 : std_logic_vector (31 downto 0);
   signal output4 : std_logic_vector (31 downto 0);         
    
   signal input1  : std_logic_vector (31 downto 0); 
   signal input2  : std_logic_vector (31 downto 0);
   signal input3  : std_logic_vector (31 downto 0);
   signal input4  : std_logic_vector (31 downto 0);         
    

   signal output1_analog : std_logic_vector (23 downto 0); 
   signal output2_analog : std_logic_vector (23 downto 0);
   signal output3_analog : std_logic_vector (23 downto 0);
   signal output4_analog : std_logic_vector (23 downto 0);         

   signal output1_8 : std_logic_vector (7 downto 0); 
   signal output2_8 : std_logic_vector (7 downto 0);
   signal output3_8 : std_logic_vector (7 downto 0);
   signal output4_8 : std_logic_vector (7 downto 0);         

begin
	
	Inst_dap_base: dap_base PORT MAP(
		clk => clk,
		res => res,
		dbi_ena => dbi_ena,
		dbi_rw => dbi_rw,
		dbi_adr => dbi_adr,
		dbi_wdata => dbi_wdata,
		dbi_rdata => dbi_rdata,
		new_sample => sampleclk,
		input1 => (others => '0'),
		input2 => (others => '0'),
		input3 => (others => '0'),
		input4 => (others => '0'),
		output1 => output1,
		output2 => output2,
		output3 => output3,
		output4 => output4
	);	
	
   resgen : process 
   begin
     res <= '0';
     wait for 1 ns;
     res <= '1';
     wait for 1 ns;
     res <= '0';  
     wait;  
   end process;	     
   
   clockgen : process 
   begin
       CLOCK_LOOP : loop
           clk <= '1';
           wait for MCLOCK_PER/2;
           clk <= '0';
           wait for MCLOCK_PER/2;
       end loop CLOCK_LOOP;    
   end process;
  
   sampleclockgen : process 
   begin
     SMPCLOCK_LOOP : loop
        sampleclk <= '0';
        wait for SMPCLOCK_PER - MCLOCK_PER;
        sampleclk <= '1';
        wait for MCLOCK_PER;
     end loop SMPCLOCK_LOOP;  
   end process;    
   
   analog_view : process
   begin
     view_loop : loop  
       output1_analog <= output1 (23 downto 0) + X"800000";  
       output2_analog <= output2 (23 downto 0) + X"800000";
       output3_analog <= output3 (23 downto 0) + X"800000";
       output4_analog <= output4 (23 downto 0) + X"800000";               
       
       output1_8 <= output1_analog (23 downto 16);
       output2_8 <= output2_analog (23 downto 16);
       output3_8 <= output3_analog (23 downto 16);
       output4_8 <= output4_analog (23 downto 16);                     
       wait for MCLOCK_PER;
     end loop;  
       
   end process;
   
   debug_process : process
     procedure debug_reset is
     begin
       dbi_ena   <= '0';
       dbi_rw    <= '0';
       dbi_adr   <= (others => '0');
       dbi_wdata <= (others => '0');
     end; 
   
     procedure write_debug_reg (       reg : in std_logic_vector ( 3 downto 0);
                                       val : in std_logic_vector ( 7 downto 0)
                               ) is    
     begin
       dbi_ena <= '1';
       dbi_rw  <= '0';
       dbi_adr <= reg;
       dbi_wdata <= val;
       wait for MCLOCK_PER;
       dbi_ena <= '0';
     end procedure;
     
     procedure read_debug_reg  (       reg : in  std_logic_vector ( 3 downto 0);
                                signal val : out std_logic_vector ( 7 downto 0)
                               ) is    
     begin
       dbi_ena <= '1';
       dbi_rw  <= '1';
       dbi_adr <= reg;
       wait for MCLOCK_PER;
       val <= dbi_rdata;
       dbi_ena <= '0';
     end procedure;
     
     procedure set_debug_mode ( onoff  : in std_logic;
                                run    : in std_logic;
                                single : in std_logic
                               ) is 
     begin
       write_debug_reg (X"0", "00000" & run & single & onoff);  
     end procedure;
     
     procedure set_breakpoint_enable (bps : in std_logic_vector (3 downto 0)
                                     ) is
     begin
       write_debug_reg (X"1", "0000" & bps);  
     end;
     
     procedure set_breakpoint1 (adr   : in std_logic_vector (11 downto 0)
                               ) is
     begin
       write_debug_reg (X"2", adr ( 7 downto 0) );
       write_debug_reg (X"3", "0000000" & adr ( 8) );       
     end procedure; 
     
   begin
     debug_reset;  
       
     wait for 1 ms;
   
     set_debug_mode ('1', '1', '0');
     set_breakpoint1 (X"030");
     set_breakpoint_enable ("0001");
     
     wait for 30 us;
     
     set_debug_mode ('1', '1', '1');
     
     wait for 30 us;
     
     set_debug_mode ('1', '1', '1');

     wait for 30 us;
     
     set_debug_mode ('1', '1', '0');
     
     
     wait for 30 ms;
     
     set_debug_mode ('1', '1', '1');
     
     wait;  
   end process;   
   
end Behavioral;
