library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity dap_base_tb2 is
end dap_base_tb2;

architecture Behavioral of dap_base_tb2 is
   

	COMPONENT dap_main
	PORT(
		clk : IN std_logic;
		res : IN std_logic;
		spi_di : IN std_logic;
		spi_cs : IN std_logic;
		spi_clk : IN std_logic;          
		outpwm : OUT std_logic;
		spi_do : OUT std_logic
		);
	END COMPONENT;


   
  constant MCLOCK_PER   : time := 20 ns;
	constant SMPCLOCK_PER : time := 20 us;  
	constant M_CLKPER_WAIT : integer := 10; 		
    
  signal clk       : std_logic;    
  signal res       : std_logic;
	signal sampleclk : std_logic;	 
	
	signal outpwm : std_logic;	 

	signal spi_cs  : std_logic;
	signal spi_clk : std_logic;
	signal spi_di  : std_logic;
	signal spi_do  : std_logic;
	
	signal spi_rdata  : std_logic_vector ( 7 downto 0);	
	signal spi_rdatat : std_logic_vector ( 7 downto 0);	

	signal memory     : std_logic_vector (31 downto 0);	

	
begin

	Inst_dap_main: dap_main PORT MAP(
		clk 		=> clk,
		res 		=> res,
		outpwm 	=> outpwm,
		spi_di 	=> spi_di,
		spi_do 	=> spi_do,
		spi_cs 	=> spi_cs,
		spi_clk => spi_clk
	);

	
   resgen : process 
   begin
     res <= '0';
     wait for 1 ns;
     res <= '1';
     wait for 1 ns;
     res <= '0';  
     wait;  
   end process;	     
   
   clockgen : process 
   begin
       CLOCK_LOOP : loop
           clk <= '1';
           wait for MCLOCK_PER/2;
           clk <= '0';
           wait for MCLOCK_PER/2;
       end loop CLOCK_LOOP;    
   end process;
  
   sampleclockgen : process 
   begin
     SMPCLOCK_LOOP : loop
        sampleclk <= '0';
        wait for SMPCLOCK_PER - MCLOCK_PER;
        sampleclk <= '1';
        wait for MCLOCK_PER;
     end loop SMPCLOCK_LOOP;  
   end process;   
   
   spi2dev : process
   
	  procedure spi_cmdwr (       datain : in std_logic_vector (7 downto 0); 
	                       signal dataout : out std_logic_vector (7 downto 0)
	                      ) is
	    variable i : integer; 
	  begin
	    for i in 7 downto 0 loop
	      wait for M_CLKPER_WAIT * MCLOCK_PER / 2;    
	      spi_clk <= not '1';		 
	      wait for M_CLKPER_WAIT * MCLOCK_PER / 2;    
	      spi_di  <= datain (i);		 
	      wait for M_CLKPER_WAIT * MCLOCK_PER / 2;    
	      spi_clk <= '1';  
	      wait for M_CLKPER_WAIT * MCLOCK_PER / 2;    
	      dataout (i) <= spi_do; 

	    end loop;  
	    wait for 4*M_CLKPER_WAIT * MCLOCK_PER / 2;    
	  end spi_cmdwr;  
	  
	  procedure spi_select is
		begin
			spi_cs <= '0';
			wait for 6 * M_CLKPER_WAIT * MCLOCK_PER;
		end procedure;

	  procedure spi_deselect is
		begin
			spi_cs <= '1';
			wait for 6 * M_CLKPER_WAIT * MCLOCK_PER;
		end procedure;		 
		
	  procedure spi_reset is
		begin
		  spi_cs  <= '1';  
		  spi_di  <= '0';
		  spi_clk <= '1';			  

		  spi_rdata	<= (others => '0');
		  spi_rdatat	<= (others => '0');

		  wait for 10 * M_CLKPER_WAIT * MCLOCK_PER;
		end procedure;		
		
		procedure dap_write_memory (adr   : in std_logic_vector (15 downto 0);
		                            value : in std_logic_vector (7 downto 0) 
		                           ) is
      begin
        spi_select;
        spi_cmdwr (X"F0", spi_rdatat);
        spi_cmdwr (X"00", spi_rdatat);
        spi_cmdwr (adr (15 downto  8), spi_rdatat);
        spi_cmdwr (adr ( 7 downto  0), spi_rdatat);
        spi_cmdwr (value, spi_rdatat);
        spi_deselect;        
      end procedure;
      
		procedure dap_read_memory (adr   : in std_logic_vector (15 downto 0);
		                           signal value : out std_logic_vector (7 downto 0) 
		                          ) is
      begin
        spi_select;
        spi_cmdwr (X"F0", spi_rdatat);
        spi_cmdwr (X"01", spi_rdatat);
        spi_cmdwr (adr (15 downto  8), spi_rdatat);
        spi_cmdwr (adr ( 7 downto  0), spi_rdatat);
        spi_cmdwr (X"00", value);
        spi_deselect;        
      end procedure;   
      
      procedure dap_write_memory_long (adr   : in std_logic_vector (15 downto 0);
		                                 value : in std_logic_vector (31 downto 0) 
                                      ) is
      begin
        dap_write_memory (adr (15 downto 12) & "0" & adr (8 downto 0) & "00", value ( 7 downto  0));
        dap_write_memory (adr (15 downto 12) & "0" & adr (8 downto 0) & "01", value (15 downto  8));
        dap_write_memory (adr (15 downto 12) & "0" & adr (8 downto 0) & "10", value (23 downto 16));
        dap_write_memory (adr (15 downto 12) & "0" & adr (8 downto 0) & "11", value (31 downto 24));
      
      end procedure; 
      
      procedure dap_read_memory_long  (adr   : in std_logic_vector (15 downto 0);
		                                 signal value : out std_logic_vector (31 downto 0) 
                                      ) is
      begin
        dap_read_memory (adr (15 downto 12) & "0" & adr (8 downto 0) & "00", value ( 7 downto  0));
        dap_read_memory (adr (15 downto 12) & "0" & adr (8 downto 0) & "01", value (15 downto  8));
        dap_read_memory (adr (15 downto 12) & "0" & adr (8 downto 0) & "10", value (23 downto 16));
        dap_read_memory (adr (15 downto 12) & "0" & adr (8 downto 0) & "11", value (31 downto 24));
      
      end procedure; 
         
		
		procedure dap_write_debug  (reg   : in std_logic_vector ( 3 downto 0);
		                            value : in std_logic_vector ( 7 downto 0) 
		                           ) is
      begin
        spi_select;
        spi_cmdwr (X"F0", spi_rdatat);
        spi_cmdwr (X"02", spi_rdatat);
        spi_cmdwr ("0000" & reg, spi_rdatat);
        spi_cmdwr (value, spi_rdatat);
        spi_deselect;        
      end procedure;
		
		procedure dap_read_debug  (reg   : in std_logic_vector ( 3 downto 0);
		                           signal value : out std_logic_vector ( 7 downto 0) 
		                          ) is
      begin
        spi_select;
        spi_cmdwr (X"F0", spi_rdatat);
        spi_cmdwr (X"03", spi_rdatat);
        spi_cmdwr ("0000" & reg, spi_rdatat);
        spi_cmdwr (X"00", value);
        spi_deselect;        
      end procedure;  
      
        
   begin
     spi_reset;
     
     wait for 200 us;
     
     --dap_write_debug (X"0", X"03"); -- dap halt
     
     wait for 40 us;
     
     --dap_read_memory_long (X"0000", memory); 
     
     wait for 5 us;
     
     --dap_write_memory_long (X"0000", X"ff102040");


     wait for 40 us;
     
     --dap_write_debug (X"0", X"04"); -- dap weiter


       
     wait;  
   end process;

   
--   analog_view : process
--   begin
--     view_loop : loop  
--       output1_analog <= output1 (23 downto 0) + X"800000";  
--       output2_analog <= output2 (23 downto 0) + X"800000";
--       output3_analog <= output3 (23 downto 0) + X"800000";
--       output4_analog <= output4 (23 downto 0) + X"800000";               
--       
--       output1_8 <= output1_analog (23 downto 16);
--       output2_8 <= output2_analog (23 downto 16);
--       output3_8 <= output3_analog (23 downto 16);
--       output4_8 <= output4_analog (23 downto 16);                     
--       wait for MCLOCK_PER;
--     end loop;  
--   end process;
   
 
   
end Behavioral;

