---------------------------------------------------------------------------------------------------
--
--  DAP_MAIN.VHDL
--
--  Top-Level Design des Gesamt-Systems
--
--  Autor : Rene Bllhoff
--  Datum : 27.09.2005
--
--  (c) 2005 by Rene Bllhoff
--
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity dap_main is
		port (	clk 		: in 	std_logic;
						res			:	in 	std_logic;

						outpwm	: out std_logic;

						spi_di  : in  std_logic;
						spi_do  : out std_logic;
						spi_cs  : in  std_logic;
						spi_clk : in  std_logic;

						pbtn		: in 	std_logic_vector ( 2 downto 0);
						swin		: in 	std_logic_vector ( 7 downto 0);
						ledout  : out	std_logic_vector ( 7 downto 0);
						segout	: out	std_logic_vector ( 7 downto 0);
						digout  : out	std_logic_vector ( 3 downto 0)
				 );
end dap_main;

architecture Behavioral of dap_main is

	COMPONENT dap_base
	PORT(
		clk : IN std_logic;
		res : IN std_logic;

		dapi_din	: IN std_logic_vector(7 downto 0);
		dapi_dout	: OUT std_logic_vector(7 downto 0);
		dapi_trg  : IN std_logic;
		dapi_sel	: IN std_logic;
		dapi_rdy	: OUT std_logic;

		new_sample : IN std_logic;
		input1 : IN std_logic_vector(31 downto 0);
		input2 : IN std_logic_vector(31 downto 0);
		input3 : IN std_logic_vector(31 downto 0);
		input4 : IN std_logic_vector(31 downto 0);          
		output1 : OUT std_logic_vector(31 downto 0);
		output2 : OUT std_logic_vector(31 downto 0);
		output3 : OUT std_logic_vector(31 downto 0);
		output4 : OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;


	COMPONENT dap_audio_route
	PORT(
		clk : IN std_logic;
		res : IN std_logic;
		outmode : IN std_logic_vector(2 downto 0);
		input1 : IN std_logic_vector(31 downto 0);
		input2 : IN std_logic_vector(31 downto 0);
		input3 : IN std_logic_vector(31 downto 0);
		input4 : IN std_logic_vector(31 downto 0);          
		outpwm : OUT std_logic;
		smp_clk : OUT std_logic;
		output1 : OUT std_logic_vector(31 downto 0);
		output2 : OUT std_logic_vector(31 downto 0);
		output3 : OUT std_logic_vector(31 downto 0);
		output4 : OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;

	COMPONENT spi_sm_stdio
	PORT(
		clk50m : IN std_logic;
		res : IN std_logic;
		fsm_select : IN std_logic;
		fsm_trigger : IN std_logic;
		spi_datain : IN std_logic_vector(7 downto 0);
		swin : IN std_logic_vector(7 downto 0);
		pbtn : IN std_logic_vector(2 downto 0);          
		fsm_ready : OUT std_logic;
		spi_dataout : OUT std_logic_vector(7 downto 0);
		ledout : OUT std_logic_vector(7 downto 0);
		segout : OUT std_logic_vector(7 downto 0);
		digout : OUT std_logic_vector(3 downto 0)
		);
	END COMPONENT;

	COMPONENT spi_dispatcher
	PORT(
		clk50m : IN std_logic;
		res : IN std_logic;
		spi_di : IN std_logic;
		spi_clk : IN std_logic;
		spi_cs : IN std_logic;
		dev1_rdy : IN std_logic;
		dev1_din : IN std_logic_vector(7 downto 0);
		dev2_rdy : IN std_logic;
		dev2_din : IN std_logic_vector(7 downto 0);
		dev3_rdy : IN std_logic;
		dev3_din : IN std_logic_vector(7 downto 0);
		dev4_rdy : IN std_logic;
		dev4_din : IN std_logic_vector(7 downto 0);
		dev5_rdy : IN std_logic;
		dev5_din : IN std_logic_vector(7 downto 0);
		dev6_rdy : IN std_logic;
		dev6_din : IN std_logic_vector(7 downto 0);          
		spi_do : OUT std_logic;
		dev_dout : OUT std_logic_vector(7 downto 0);
		dev1_sel : OUT std_logic;
		dev1_trg : OUT std_logic;
		dev2_sel : OUT std_logic;
		dev2_trg : OUT std_logic;
		dev3_sel : OUT std_logic;
		dev3_trg : OUT std_logic;
		dev4_sel : OUT std_logic;
		dev4_trg : OUT std_logic;
		dev5_sel : OUT std_logic;
		dev5_trg : OUT std_logic;
		dev6_sel : OUT std_logic;
		dev6_trg : OUT std_logic;
		dport : OUT std_logic_vector(3 downto 0);
		scstate : OUT std_logic_vector(3 downto 0);
		bitstate : OUT std_logic_vector(3 downto 0)
		);
	END COMPONENT;


	signal output1_32 : std_logic_vector (31 downto 0);
	signal output2_32 : std_logic_vector (31 downto 0);
	signal output3_32 : std_logic_vector (31 downto 0);
	signal output4_32 : std_logic_vector (31 downto 0);

	signal newsample	: std_logic;



	signal spi_dout			: std_logic_vector (7 downto 0);
	signal dap_din			: std_logic_vector (7 downto 0);
	signal dap_trg 			: std_logic;
	signal dap_rdy 			: std_logic;
	signal dap_sel 			: std_logic;

	signal std_din			: std_logic_vector (7 downto 0);
	signal std_trg 			: std_logic;
	signal std_rdy 			: std_logic;
	signal std_sel 			: std_logic;



begin

	Inst_dap_base: dap_base PORT MAP(
		clk 				=> clk,
		res 				=> res,
		dapi_din		=> spi_dout,
		dapi_dout		=> dap_din,
		dapi_trg    => dap_trg,
		dapi_sel		=> dap_sel,
		dapi_rdy		=> dap_rdy,
		new_sample 	=> newsample,
		input1 			=> (others => '0'),
		input2 			=> (others => '0'),
		input3 			=> (others => '0'),
		input4 			=> (others => '0'),
		output1 		=> output1_32,
		output2 		=> output2_32,
		output3 		=> output3_32,
		output4 		=> output4_32
	);


	Inst_dap_audio_route: dap_audio_route PORT MAP(
		clk 				=> clk,
		res 				=> res,
		outpwm 			=> outpwm,
		smp_clk 		=> newsample,
		outmode 		=> "000",
		input1 			=> output1_32,
		input2 			=> output2_32,
		input3 			=> output3_32,
		input4 			=> output4_32,
		output1 		=> open,
		output2 		=> open,
		output3 		=> open,
		output4 		=> open
	);

	Inst_spi_sm_stdio: spi_sm_stdio PORT MAP(
		clk50m 			=> clk,
		res 				=> res,
		fsm_select 	=> std_sel,
		fsm_trigger => std_trg,
		fsm_ready 	=> std_rdy,
		spi_datain 	=> spi_dout,
		spi_dataout => std_din,
		swin 				=> swin,
		pbtn 				=> pbtn,
		ledout 			=> ledout,
		segout 			=> segout,
		digout 			=> digout
	);


	Inst_spi_dispatcher: spi_dispatcher PORT MAP(
		clk50m 			=> clk,
		res 				=> res,
		spi_do 			=> spi_do,
		spi_di 			=> spi_di,
		spi_clk 		=> spi_clk,
		spi_cs 			=> spi_cs,
		dev_dout 		=> spi_dout,
		dev1_sel 		=> dap_sel,
		dev1_rdy 		=> dap_rdy,
		dev1_trg 		=> dap_trg,
		dev1_din 		=> dap_din,
		dev2_sel 		=> std_sel,
		dev2_rdy 		=> std_rdy,
		dev2_trg 		=> std_trg,
		dev2_din 		=> std_din,
		dev3_sel 		=> open,
		dev3_rdy 		=> '1',
		dev3_trg 		=> open,
		dev3_din 		=> (others => '0'),
		dev4_sel 		=> open,
		dev4_rdy 		=> '1',
		dev4_trg 		=> open,
		dev4_din 		=> (others => '0'),
		dev5_sel 		=> open,
		dev5_rdy 		=> '1',
		dev5_trg 		=> open,
		dev5_din 		=> (others => '0'),
		dev6_sel 		=> open,
		dev6_rdy 		=> '1',
		dev6_trg 		=> open,
		dev6_din 		=> (others => '0'),
		dport	 			=> open,
		scstate 		=> open,
		bitstate 		=> open
	);


end Behavioral;

