---------------------------------------------------------------------------------------------------
--
--  SEG7MUX.VHDL
--
--  Multiplexer fr das 7-Segment Display
--
--  Autor : Rene Bllhoff
--  Datum : 21.05.2005
--
--  (c) 2005 by Rene Bllhoff
--
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity seg7mux is
  port (
		clk1k   : in 	std_logic;
		res			:	in	std_logic;
		value16	: in 	std_logic_vector (15 downto 0);
		dots4		: in  std_logic_vector ( 3 downto 0);
		segout  : out	std_logic_vector ( 7 downto 0);
		digout  : out	std_logic_vector ( 3 downto 0)
	);
end seg7mux;

architecture Behavioral of seg7mux is
  signal 	strobe 	: std_logic_vector (1 downto 0);
  signal 	val4   	: std_logic_vector (3 downto 0);
	signal	dp			: std_logic;
begin
  process (clk1k, res, value16, dots4, strobe)
	begin
		if res = '1' then
			strobe 	<= (others => '0');
			val4	 	<= (others => '0');
			segout 	<= (others => '1');
			digout 	<= (others => '1');
			dp			<= '1';
		else
			if clk1k'event and clk1k = '1' then
				strobe <= strobe + '1';
			end if;

			case strobe is
			  when "00" 	=> val4 	<= value16 ( 3 downto  0); 	digout <= "1110";	dp <= not dots4 (0);
				when "01"	 	=> val4 	<= value16 ( 7 downto  4); 	digout <= "1101";	dp <= not dots4 (1);
				when "10" 	=> val4 	<= value16 (11 downto  8); 	digout <= "1011";	dp <= not dots4 (2);
				when "11" 	=> val4	 	<= value16 (15 downto 12); 	digout <= "0111";	dp <= not dots4 (3);
				when others	=> val4	 	<= value16 (15 downto 12); 	digout <= "0111";	dp <= not dots4 (3);
			end case;


			case val4 is
			  when "0000" => segout <= "0000001" & dp;	 	-- 0
				when "0001" => segout <= "1001111" & dp;	 	-- 1
				when "0010" => segout <= "0010010" & dp;		-- 2
				when "0011" => segout <= "0000110" & dp;		-- 3
			  when "0100" => segout <= "1001100" & dp;		-- 4
				when "0101" => segout <= "0100100" & dp;		-- 5
				when "0110" => segout <= "0100000" & dp;		-- 6
				when "0111" => segout <= "0001111" & dp;		-- 7
			  when "1000" => segout <= "0000000" & dp;		-- 8
				when "1001" => segout <= "0000100" & dp;		-- 9
				when "1010" => segout <= "0001000" & dp;		-- a
				when "1011" => segout <= "1100000" & dp;		-- b
			  when "1100" => segout <= "0110001" & dp;		-- c
				when "1101" => segout <= "1000010" & dp;		-- d
				when "1110" => segout <= "0110000" & dp;		-- e
				when "1111" => segout <= "0111000" & dp;		-- f
				when others => segout <= "1111111" & dp;		-- 0
			end case;

		end if;

	end process;

end Behavioral;
