---------------------------------------------------------------------------------------------------
--
--  SPI_CMD.VHDL
--
--  Senden und Empfangen von Bytes auf der SPI-Schnittstelle
--
--  Autor : Rene Bllhoff
--  Datum : 27.09.2005
--
--  (c) 2005 by Rene Bllhoff
--
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;


entity spi_cmd is
    generic (
      db_width : integer := 32
    );
    
    port (
        clk50m  			: in  std_logic;
        res     			: in  std_logic;        
        
        spi_cs  			: in  std_logic;
        spi_clk 			: in  std_logic;
        spi_di  			: in  std_logic;
        spi_do  			: out std_logic;                        
        
				busy					: out std_logic;

        dat2host      : in  std_logic_vector (db_width - 1 downto 0);
				dat2host_req  : in  std_logic;

        dat2dev       : out std_logic_vector (db_width - 1 downto 0);
				dat2dev_rcv   : out std_logic
         );
end entity spi_cmd;


         
architecture behave of spi_cmd is
    
  signal spi_state  : std_logic_vector (2 downto 0);

  signal spi_clk_z1	: std_logic;
  signal spi_clk_z2	: std_logic;
  signal spi_clk_z3	: std_logic;
 	signal spi_cs_z1	: std_logic;
  signal spi_di_z1	: std_logic;

	signal spi_rcvd		: std_logic;
	
	signal spi_ledge : std_logic;
    
  signal data_send  : std_logic_vector  (db_width downto 0); 
	signal data_recv  : std_logic_vector (db_width downto 0); 

	signal spi_busy		: std_logic;
	signal spi_host_ack : std_logic;

begin
   
  process (clk50m, res, spi_state, 
                        spi_cs, 
                        spi_clk, 
                        spi_di, 
                        spi_clk_z1, 
                        spi_clk_z2, 
                        spi_clk_z3, 
                        spi_cs_z1, 
                        data_send
                        )
  begin
    -- RESET

    if res = '1' then
      
			-- intern reset  
			spi_clk_z1  <= '1';
			spi_clk_z2  <= '1';
			spi_clk_z3  <= '1';
			spi_cs_z1     <= '1';

			spi_rcvd		  <= '0';
			spi_do      <= '1';
			
			spi_busy				<= '0';
			spi_ledge   <= '0';
			
	      data_send 		<= (others => '0');
			data_recv 		<=  ( 0 => '1', others => '0');
											
			dat2dev     <= (others => '0');
			spi_state		 <= (others => '0');

 	    -- extern reset       
        
      dat2dev_rcv  <= '0';

      spi_do <= '1';  

		-- CLOCK

    elsif clk50m'event and clk50m = '1' then

      -- SPI HANDLING
			case spi_state is
				when "000"  => if spi_cs_z1 = '0' then
										spi_state 	  <= "001";
										data_recv    <= ( 0 => '1', others => '0');
									else
										spi_state 		<= "000";
										spi_do      <= '1';
										spi_busy    <= '0';
									end if;
			  when "001"   => if    spi_clk_z3 = '0' and spi_clk_z2 = '0' and spi_clk_z1 = '1' then
										spi_state 			<= "010";	
										data_recv		<= data_recv (db_width - 1 downto 0) & spi_di_z1;
										spi_rcvd			<= '0';	
										spi_busy					<= '1';
										spi_ledge  <= '1';
									elsif spi_clk_z3 = '1' and spi_clk_z2 = '1' and spi_clk_z1 = '0' then
										spi_state 		<= "010";
										spi_do    		<= data_send (db_width - 1);
										spi_rcvd			<= '0';
										spi_busy					<= '1';
										spi_ledge  <= '0';
									end if;
									if spi_cs_z1 = '1' then
									   spi_state <= "000";
									end if;		 
											 
				when "010"  => if data_recv (db_width) = '1' and spi_ledge = '1' then
									   spi_state 		<= "011";
									else
									   spi_state 		<= "001";	 	
									end if;
									if spi_ledge = '0' then		 
  							         data_send 			<= data_send (db_width - 1 downto 0) & '0';	
  							      end if;  
									if spi_cs_z1 = '1' then
									   spi_state <= "000";
									end if;		 
				when "011"  => dat2dev     		<= data_recv (db_width - 1 downto 0); 
									spi_rcvd 			<= '1';
									spi_state 	 		<= "000";
									spi_busy						<= '0';												 
									if spi_cs_z1 = '1' then
									   spi_state <= "000";
									end if;		 
				when others => spi_state   		<= "000";
			end case;

			-- clock detection
 		  spi_clk_z1 <= spi_clk;
 		  spi_clk_z2 <= spi_clk_z1;
 		  spi_clk_z3 <= spi_clk_z2;
			spi_cs_z1  <= spi_cs;
			spi_di_z1  <= spi_di;

		  -- INTERFACE

 		  -- send request ?
			if dat2host_req = '1' and spi_busy = '0' and spi_host_ack = '0' then
			   data_send (db_width - 1 downto 0)	<= dat2host;
				spi_host_ack 	<= '1';  	-- bernahme signalisieren
			else 
            spi_host_ack 	<= '0';		-- aktiv oder keine daten		  
			end if;

         busy        <= spi_busy;  
         dat2dev_rcv <= spi_rcvd;

  	end if;

  end process;  

end architecture behave;