---------------------------------------------------------------------------------------------------
--
--  SPI_DISPATCHER.VHDL
--
--  Dispatcher fr Daten und Steuerleitungen der SPI-Entity
--
--  Autor : Rene Bllhoff
--  Datum : 27.09.2005
--
--  (c) 2005 by Rene Bllhoff
--
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity spi_dispatcher is

    Port ( clk50m 	: in	 	std_logic;
           res 			: in	 	std_logic;

					 spi_do 	: out 	std_logic;
					 spi_di 	: in 		std_logic;
					 spi_clk 	: in 		std_logic;
					 spi_cs 	: in  	std_logic;

					 dev_dout	: out std_logic_vector (7 downto 0);  -- data in  aus sicht des devices


					 dev1_sel	 : out std_logic;
					 dev1_rdy	 : in  std_logic;
					 dev1_trg	 : out std_logic;
					 dev1_din	 : in  std_logic_vector (7 downto 0);	 -- data out aus sicht des devices

					 dev2_sel	 : out std_logic;
					 dev2_rdy	 : in  std_logic;
					 dev2_trg	 : out std_logic;
					 dev2_din	 : in  std_logic_vector (7 downto 0);

					 dev3_sel	 : out std_logic;
					 dev3_rdy	 : in  std_logic;
					 dev3_trg  : out std_logic;
					 dev3_din	 : in  std_logic_vector (7 downto 0);

					 dev4_sel	 : out std_logic;
					 dev4_rdy	 : in  std_logic;
					 dev4_trg  : out std_logic;
					 dev4_din	 : in  std_logic_vector (7 downto 0);

					 dev5_sel	 : out std_logic;
					 dev5_rdy	 : in  std_logic;
					 dev5_trg  : out std_logic;
					 dev5_din  : in  std_logic_vector (7 downto 0);

					 dev6_sel	 : out std_logic;
					 dev6_rdy	 : in  std_logic;
					 dev6_trg  : out std_logic;
					 dev6_din	 : in  std_logic_vector (7 downto 0);


					 dport		 : out std_logic_vector (3 downto 0);
					 scstate	 : out std_logic_vector (3 downto 0);
					 bitstate	 : out std_logic_vector (3 downto 0)
				 );

end spi_dispatcher;

architecture Behavioral of spi_dispatcher is

	component spi_cmd is
    generic (
      db_width : integer := 8
    );
    
    port (
        clk50m  			: in  std_logic;
        res     			: in  std_logic;        
        
        spi_cs  			: in  std_logic;
        spi_clk 			: in  std_logic;
        spi_di  			: in  std_logic;
        spi_do  			: out std_logic;                        
        
				busy					: out std_logic;

        dat2host     	: in  std_logic_vector (db_width - 1 downto 0);
				dat2host_req 	: in  std_logic;

        dat2dev      	: out std_logic_vector (db_width - 1 downto 0);
				dat2dev_rcv  	: out std_logic
         );
	end component;

	component edge_detect is 
				port (
					clk50m 	: in  std_logic;
	        res    	: in  std_logic; 

					edge 		: in  std_logic;
					i 			: in  std_logic;
					o 			: out std_logic  
				);
  end component; 



	--signal dots4 : std_logic_vector (3 downto 0);

	signal spi_datain     : std_logic_vector (7 downto 0);
	signal spi_dataout    : std_logic_vector (7 downto 0);

	signal spi_trigger : std_logic;
	signal fsm_trigger : std_logic;

	signal spi_cmd_state  : std_logic_vector (3 downto 0);
	signal data_route     : std_logic_vector (3 downto 0);
	signal spi_ctrl       : std_logic_vector (3 downto 0);
		
	signal fsm_is_ready   : std_logic;

	signal spi_cs_z1      : std_logic;
	signal spi_cs_z2      : std_logic;
	signal spi_cs_z3      : std_logic;


begin

  
	spi_cmd1 : spi_cmd port map (
				        clk50m  => clk50m,
				        res     => res,     
        
				        spi_cs  => spi_cs,
				        spi_clk => spi_clk,
				        spi_do  => spi_do,
				        spi_di  => spi_di,
        
								busy		=> open,

				        dat2host     => spi_dataout,
								dat2host_req => '1',

				        dat2dev      => spi_datain,
								dat2dev_rcv  => spi_trigger
		          );
	
	spi_cmd_egde : edge_detect port map (
								clk50m => clk50m, 
  			        res     => res,  

								edge => '1',
								i => spi_trigger,
								o => fsm_trigger
							);


  process (	clk50m, res, 
						fsm_trigger, spi_cmd_state, spi_datain, fsm_is_ready,	data_route, 
						dev1_rdy, dev1_din, dev2_rdy, dev2_din, dev3_rdy, dev3_din, 
						dev4_rdy, dev4_din, dev5_rdy, dev5_din, dev6_rdy, dev6_din
					)
	begin
    if res = '1' then
			
			spi_cmd_state		<= (others => '0');
			data_route			<= (others => '0');
			spi_ctrl        <= (others => '0');


			dev1_sel				<= '0';
			dev1_trg  			<= '0';

			dev2_sel				<= '0';
			dev2_trg  			<= '0';
			
			dev3_sel				<= '0';
			dev3_trg  			<= '0';


			dev4_sel				<= '0';
			dev4_trg  			<= '0';

			dev5_sel				<= '0';
			dev5_trg  			<= '0';
			
			dev6_sel				<= '0';
			dev6_trg  			<= '0';


			fsm_is_ready  	<= '1';

			spi_cs_z1				<= '1';
			spi_cs_z2				<= '1';
			spi_cs_z3				<= '1';

		elsif clk50m'event and clk50m = '1' then

			spi_cs_z1				<= spi_cs;
			spi_cs_z2				<= spi_cs_z1;
			spi_cs_z3				<= spi_cs_z2;

			if spi_cs_z3 = '0' then
				if fsm_trigger = '1' then
				  case spi_cmd_state is
						when X"0" => 	 case spi_datain (7 downto 0) is
													   when X"F0"   => spi_cmd_state <= X"1"; data_route <= X"1"; 
													   when X"F1"   => spi_cmd_state <= X"1"; data_route <= X"2"; 
													   when X"F2"   => spi_cmd_state <= X"1"; data_route <= X"3"; 
													   when X"F3"   => spi_cmd_state <= X"1"; data_route <= X"4"; 
													   when X"F4"   => spi_cmd_state <= X"1"; data_route <= X"5"; 
													   when X"F5"   => spi_cmd_state <= X"1"; data_route <= X"6"; 
														when others   => spi_cmd_state <= X"0"; data_route <= X"0";
													 end case; 	
						when X"1" => NULL;						 
						when others => spi_cmd_state <= X"0";
					end case;
	      end if;

	      case spi_ctrl is
	          when X"0" => if data_route /= X"0" then		 -- warte auf fsm_ready = 0 (wenn data route gesetzt)
	                          if fsm_is_ready = '0' then
	                            spi_ctrl <= X"1";  
	                          end if;
	                        end if;
	          when X"1" => if data_route /= X"0" then		 -- warte auf fsm_ready = 1 (wenn data route gesetzt) -> reset
	                          if fsm_is_ready = '1' then
	                            spi_ctrl <= X"0";  
				                   		spi_dataout <= (others => '0');
					                		spi_cmd_state <= X"0"; 
					                		data_route 		<= X"0";
	                          end if;
	                        end if;
	          when others => NULL;
	       end case;    
       
				--dots4 (0) <= fsm_stdio_sel;
			else
			  data_route <= X"0"; spi_cmd_state <= X"0"; spi_ctrl <= X"0";
			end if;
		end if;

       -- spi - device - mux
		case data_route is
			when X"1" 	 => dev1_sel <= '1'; 
											dev2_sel <= '0'; 
											dev3_sel <= '0'; 
											dev4_sel <= '0';
											dev5_sel <= '0';
											dev6_sel <= '0';

											dev1_trg		 <= fsm_trigger; 
			                fsm_is_ready <= dev1_rdy; 
			                spi_dataout  <= dev1_din;

			when X"2" 	 => dev1_sel <= '0'; 
											dev2_sel <= '1'; 
											dev3_sel <= '0'; 
											dev4_sel <= '0';
											dev5_sel <= '0';
											dev6_sel <= '0';

			                dev2_trg 		 <= fsm_trigger; 
			                fsm_is_ready <= dev2_rdy; 
			                spi_dataout  <= dev2_din;

			when X"3" 	 => dev1_sel <= '0'; 
											dev2_sel <= '0'; 
											dev3_sel <= '1'; 
											dev4_sel <= '0';
											dev5_sel <= '0';
											dev6_sel <= '0';

			                dev3_trg <= fsm_trigger; 
			                fsm_is_ready <= dev3_rdy; 
			                spi_dataout  <= dev3_din;
			               
			when X"4" 	 => dev1_sel <= '0'; 
											dev2_sel <= '0'; 
											dev3_sel <= '0'; 				                
											dev4_sel <= '1';
											dev5_sel <= '0';
											dev6_sel <= '0';

											dev4_trg		 <= fsm_trigger; 
			                fsm_is_ready <= dev4_rdy; 
			                spi_dataout  <= dev4_din;

			when X"5" 	 => dev1_sel <= '0'; 
											dev2_sel <= '0'; 
											dev3_sel <= '0'; 
											dev4_sel <= '0';
											dev5_sel <= '1';
											dev6_sel <= '0';

			                dev5_trg 		 <= fsm_trigger; 
			                fsm_is_ready <= dev5_rdy; 
			                spi_dataout  <= dev5_din;

			when X"6" 	 => dev1_sel <= '0'; 
											dev2_sel <= '0'; 
											dev3_sel <= '0'; 
											dev4_sel <= '0';
											dev5_sel <= '0';
											dev6_sel <= '1';

			                dev6_trg <= fsm_trigger; 
			                fsm_is_ready <= dev6_rdy; 
			                spi_dataout  <= dev6_din;
			               
			when others  => dev1_sel <= '0';                               
			                dev2_sel <= '0'; 
											dev3_sel <= '0'; 
											dev4_sel <= '0';
											dev5_sel <= '0';
											dev6_sel <= '0';
											
											fsm_is_ready <= '1'; 					     
			                spi_dataout  <= (others => '0');

		end case;	
			



	end process;


	dport <= data_route;
  scstate <= spi_cmd_state;

	dev_dout <= spi_datain;

end Behavioral;
