---------------------------------------------------------------------------------------------------
--
--  SPI_SM_STDIO.VHDL
--
--  SPI-Interfacing Statemachine fr Onboard-Standard-Hardware (LEDs/7Segment/Schalter/Buttons)
--
--  Autor : Rene Bllhoff
--  Datum : 27.09.2005
--
--  (c) 2005 by Rene Bllhoff
--
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity spi_sm_stdio is
    port (
        clk50m  		: in  std_logic;
        res     		: in  std_logic;  
				
			--	dots4				: in std_logic_vector (3 downto 0);    

				fsm_select  : in  std_logic;
				fsm_trigger : in  std_logic;
				fsm_ready		: out std_logic;

				spi_datain  : in  std_logic_vector (7 downto 0);
				spi_dataout : out std_logic_vector (7 downto 0);

				swin				:	in	 	std_logic_vector ( 7 downto 0);
				pbtn				:	in	 	std_logic_vector ( 2 downto 0);	
				ledout			:	out 	std_logic_vector ( 7 downto 0);
				segout			:	out 	std_logic_vector ( 7 downto 0);
				digout			:	out 	std_logic_vector ( 3 downto 0)
		);
end spi_sm_stdio;



architecture Behavioral of spi_sm_stdio is


	component seg7mux   port (
							clk1k   : in 	std_logic;
							res			:	in	std_logic;
							value16	: in 	std_logic_vector (15 downto 0);
							dots4		: in  std_logic_vector ( 3 downto 0);
							segout  : out	std_logic_vector ( 7 downto 0);
							digout  : out	std_logic_vector ( 3 downto 0)
						);
  end component;

	component clockgen   port (
							clk50m  : in  std_logic;
							res			: in 	std_logic;
							clk1mo	: out std_logic;
							clk10ko	: out std_logic;
							clk1ko	: out std_logic;
							clk100o	: out std_logic;
							clk1o		: out std_logic
						);	
	end component;


	signal clock1k  : std_logic;
	signal clock1m  : std_logic;

	signal data16  : std_logic_vector (15 downto 0);
	signal leds		 : std_logic_vector ( 7 downto 0);	
	signal dots4s  : std_logic_vector ( 3 downto 0);

	signal spi_cmd_state : std_logic_vector ( 3 downto 0);

begin


	clockgeneration : clockgen 				port map (
											clk50m 	=> clk50m,
											res			=> res,
											clk1ko	=> clock1k,
											clk1mo	=> clock1m
 										);

	seg7muxer : seg7mux port map (
								clk1k 	=> clock1k, 
								res 		=> res, 
								value16 => data16,
								dots4 	=> dots4s,
								segout 	=> segout, 
								digout 	=> digout
							);


--  dots4s <= dots4;
  

  process (clk50m, res, 
					 fsm_trigger, fsm_select, spi_cmd_state, 
					 swin, pbtn, spi_datain)
	begin
    if res = '1' then
			
			spi_dataout 	<= (others => '0');								
			spi_cmd_state <= x"0";
			fsm_ready 		<= '1';


			data16 				<= (others => '0');
			leds	 				<= (others => '0');
			dots4s 				<= (others => '0');

		elsif clk50m'event and clk50m = '1' then

      if fsm_select = '1' then
         fsm_ready  <= '0';

				 if fsm_trigger = '1' then

					 case spi_cmd_state	is
						 when X"0" => 	case spi_datain (7 downto 0) is
							 								when X"00" => spi_cmd_state <= X"1"; fsm_ready <= '0';
															when X"01" => spi_cmd_state <= X"2"; fsm_ready <= '0';
															when X"02" => spi_cmd_state <= X"3"; fsm_ready <= '0';
															when X"03" => spi_cmd_state <= X"4"; fsm_ready <= '0';
															when X"04" => spi_cmd_state <= X"5"; fsm_ready <= '0'; spi_dataout <= swin;
															when X"05" => spi_cmd_state <= X"6"; fsm_ready <= '0'; spi_dataout <= X"0" & '0' & pbtn;
															when others=> spi_cmd_state <= X"0"; fsm_ready <= '1'; spi_dataout <= (others => '0');
							 							end case;
						 when X"1" => data16 (15 downto 8) <= spi_datain; fsm_ready <= '1'; spi_cmd_state <= X"0";
						 when X"2" => data16 ( 7 downto 0) <= spi_datain; fsm_ready <= '1'; spi_cmd_state <= X"0";
						 when X"3" => dots4s  <= spi_datain (3 downto 0); fsm_ready <= '1'; spi_cmd_state <= X"0";
						 when X"4" => leds                 <= spi_datain; fsm_ready <= '1'; spi_cmd_state <= X"0";
						 when X"5" =>                                     fsm_ready <= '1'; spi_cmd_state <= X"0";
						 when X"6" =>                                     fsm_ready <= '1'; spi_cmd_state <= X"0";
						 when others => spi_dataout <= (others => '0');		fsm_ready <= '1';	spi_cmd_state <= X"0";
					 end case;

         end if;  

      else
        fsm_ready <= '1';
			  spi_cmd_state <= X"0";
			  spi_dataout <= (others => '0');
			end if;

			ledout <= leds;
   
    end if; 

	end process;

end Behavioral;
